from twisted.internet.protocol import Protocol, ProcessProtocol, Factory
from struct import *
from twisted.python import log
from logging import DEBUG
import logging

from safe.messages import *
from safe.exceptions import *

PROCESS_TERMINATE_SIGNAL = "KILL"
STDIN = 0
STDOUT = 1
STDERR = 2
SAMPLES = 3

class NS3Process(ProcessProtocol):
    """This class manages the simulation process.

    The class extends the ProcessProtocol, and allows us to send data to and
    from a process, in this case the simulation process.

    """

    def __init__(self, xml_config, client):
        self.xml = xml_config
        self.client = client
        self.complete = False
        self.data = ""

    def connectionMade(self):
        """Called when the simulation is started."""

        log.msg("Simulation started",logLevel=DEBUG)
        self.transport.write(self.xml)
        self.transport.closeStdin()

    def processExited(self,reason):
        """Called when the simulation terminates.

        In this method we also request the next simulation to run from the
        manager.

        """

        log.msg("Simulation terminated with status %s"\
                % (reason.value.exitCode,), logLevel=DEBUG)

        self.client.nextSim()
        self.complete = True

    def childDataReceived(self, childFD, data):
        if childFD == STDOUT:
            log.msg(data)
        elif childFD == STDERR:
            log.err("Simulator Error: "+data)
        elif childFD == SAMPLES:
            log.msg("SAMPLE: "+data)

            self.data += data

            while True:
                msg, self.data = decode(self.data)
                if type(msg) == Sample:
                    self.client.sendSample(msg)
                elif msg == None:
                    return
                else:
                    raise UnknownMessageError("Invalid message")

    def endSimulation(self):
        self.transport.signalProcess(PROCESS_TERMINATE_SIGNAL)
