class UnknownMessageError(Exception):
    """An error class for handling unexpected messages which cannot be handled
    properly.

    """
    def __init__(self,value):
        self.value = value

    def __str__(self):
        return str(self.value)

class MessageDecodeError(Exception):
    """An error class for handling errors in the decoding of messages."""

    def __init__(self,value):
        self.value = value

    def __str__(self):
        return str(self.value)

class ClientProtocolAlreadyExists(Exception):
    """
    An exception class for handling the case when the simulation client factory
    is told to create more than one simulation client protocol.

    """

    def __init__(self,value):
        self.value = value

    def __str__(self):
        return str(self.value)

class ClientProtocolDoesntExist(Exception):
    """
    An exception raised when the factory is told to send a message on a
    non-existent SimulationClientProtocol.

    """

    def __init__(self,value):
        self.value = value

    def __str__(self):
        return str(self.value)

class DatabaseError(Exception):
    """
    An exception raised when a database operation does not complete as
    expected.

    """

    def __init__(self,value):
        self.value = value

    def __str__(self):
        return str(self.value)

class InvalidArgumentsException(Exception):
    """
    An exception raised when an invalid combination of arguments are given.

    """

    def __init__(self,value):
        self.value = value

    def __str__(self):
        return str(self.value)

class XMLParsingError(Exception):
    """An exception raised when an XML parsing error occurs."""

    def __init__(self,value):
        self.value = value

    def __str__(self):
        return str(self.value)

class ExecutionManagerError(Exception):
    """An exception in execution manager behavior."""

    def __init__(self,value):
        self.value = value

    def __str__(self):
        return str(self.value)

class BooleanOperatorError(Exception):
    """An invalid boolean operator exception."""

    def __init__(self,value):
        self.value = value

    def __str__(self):
        return str(self.value)

class UndeclaredFactorException(Exception):
    """An undefined experiment factor exception."""

    def __init__(self, factor_name = None, msg = None):
        self.factor_name = str(factor_name)
        self.msg = str(msg)

    def __str__(self):
        return self.msg

class MissingListException(Exception):
    """An exception raised when an experiment description document
    references a list id or filename that does not exist"""

    def __init__(self, list_id = None, msg = None):
        self.list_id = list_id
        self.msg = str(msg)

    def __str__(self):
        return self.msg

class InvalidTestStringException(Exception):
    """An exception raised when an experiment description document
    references a list id or filename that does not exist"""

    def __init__(self, test_string = None, msg = None):
        self.test_string = test_string
        self.msg = str(msg)

    def __str__(self):
        return self.msg
