from paver.easy import *
from paver.path import *

options (
        setup = dict(
            name = "SAFE",
            author = "Bryan C. Ward",
            author_email = "bryan.ward@bucknell.edu"
            )
        )

def lc(files):
    return sum([len(f.lines()) for f in files])

@task
def lines():
    """count the lines of code"""

    lines = []

    src_lines = lc(path('.').glob("*.py"))
    lines.append(src_lines)
    print "EEM LOC:\t%d" % (src_lines)

    web_lines = sum([len(x.lines()) for x in path('./webmanager').glob("*.py")])
    web_lines += sum(
        [len(x.lines()) for x in path('./webmanager/public/').glob("*.css")])
    lines.append(web_lines)
    print "Web LOC:\t%d" % (web_lines)

    test_lines = sum([len(x.lines()) for x in path('./test').glob("*.py")])
    lines.append(test_lines)
    print "Test LOC:\t%d" % (test_lines)

    migration_lines = lc(path('./migrations').glob("*.py"))
    lines.append(migration_lines)
    print "Migration LOC:\t%d" % (migration_lines)

    doc_lines = lc(path('./doc').glob("*.rst"))
    lines.append(doc_lines)
    print "Docs LOC:\t%d" % (doc_lines)

    print "--------------------"
    print "Total LOC:\t%d" % (sum(lines))

@task
def clean():
    """remove superfluous files"""
    for f in path('.').glob("*.pyc"):
        sh("rm -f %s" % (f))

    for f in path('./test').glob("*.pyc"):
        sh("rm -f %s" % (f))
    
    for f in path('./webmanager').glob("*.pyc"):
        sh("rm -f %s" % (f))

    for f in path('./xmltools').glob("*.pyc"):
        sh("rm -f %s" % (f))

    for d in path('.').glob('_trial*'):
        d.rmtree()

    path('doc/_build/').rmtree()

@task
@consume_args
def test():
    """execute tests"""
    from ConfigParser import ConfigParser
    config = ConfigParser()
    config.read("./config/server")
    db = config.get("TEST","dburi")
    sh('yoyo-migrate apply ./migrations/ %s --batch --no-cache' % (db))
    sh('trial %s test ' % ''.join(options.args))

@task
@cmdopts([('environment=', 'e','database to migrate')])
def migrate(options,info):
    """migrate the database"""
    from ConfigParser import ConfigParser

    config = ConfigParser()
    config.read("./config/server")

    if "environment" in options.migrate:
        db = config.get(options.migrate.environment,"dburi")
    else:
        db = config.get("DEVELOPMENT","dburi")

    sh('yoyo-migrate apply ./migrations/ %s --no-cache' % (db))

@task
#@cmdopts([('start','start the server'),('stop','stop the server')])
def webmanager(options,info):
    """Start the web management system for SAFE."""
    sh('twistd -ny webmanager.tac')

@task
def pyxbgen(options, info):
    sh("pyxbgen -u xml/Configuration.xsd -m xmltools.expconfig")
    sh("pyxbgen -u xml/Model.xsd -m xmltools.model")
    sh("pyxbgen -u xml/Simulation.xsd -m xmltools.siminstance")
    sh("pyxbgen -u xml/LevelList.xsd -m xmltools.levellist")
    sh("pyxbgen -u xml/Experiment.xsd -m xmltools.experiment")
    sh("pyxbgen -u xml/Template.xsd -m xmltools.template")
