#
# Copyright (c) 2012 Bucknell University
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation;
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Author: L. Felipe Perrone (perrone@bucknell.edu)
#
#!/usr/bin/perl

if ($#ARGV < 0) {
    die("Usage: extractAODV expGroupFile");
}

$expGroupFile = $ARGV[0];

open(GROUPS, "<$expGroupFile") ||
    die("Error: cannot open file $expGroupFile.");

while($line = <GROUPS>) {
    chop($line);

    push(@allgroups, $line);

    @parsed_line = split(/-/,$line);

    $exp_name = $parsed_line[0];
    $num_nodes = $parsed_line[1];
    $attack_prob = $parsed_line[2];
    $mob_seed = $parsed_line[3]; 
    $bit_rate = $parsed_line[4];
    $sess_length = $parsed_line[5];
    $packet_size = $parsed_line[6];
    $attack_on = $parsed_line[7];
    $attack_off = $parsed_line[8];

    for ($i=0; $i<10; $i++) {
	$file_schema = "$exp_name-$i-$num_nodes-$attack_prob-$mob_seed-appseed-$bit_rate-$sess_length-$packet_size-$attack_on-$attack_off";

	push(@name_schema, $file_schema);
    }
}

close(GROUPS);

#--------------------------------------------------------------------

#
# this generates the names of all the files containing experiments
# that belong to the group file passed as input parameter to this script
#
#

opendir(THISDIR, ".");
@allfiles = readdir(THISDIR);
closedir(THISDIR);

foreach $f (@allfiles) {
    foreach $x (@name_schema) {
	if ($x =~ /^(.*)-appseed-(.*)$/) {
	    $prefix = $1;
	    $suffix = $2;
	    if ($f =~ /^$prefix-([0-9]+)-$suffix$/) {
		push(@toprocess, $f);
	    }
	}
    }
}

#--------------------------------------------------------------------

# want to go through all the runs of one experiment and produce the
# stats (mean, stdev) for that experiment

while (@toprocess) {

    $control_msgs = 0;
    $data_msgs = 0;
    
    for ($i=0; $i<10; $i++) {
	$f = shift(@toprocess);

	($control_msgs, $data_msgs) = &processAODV($f);
	push(@control_samples, $control_msgs);
	push(@data_samples, $data_msgs);
    }

    foreach $s (@control_samples) {
	print "$s\n";
    }

    print "$f -- $#control_samples\n";
    print "control messages = ", &computeStats(@control_samples), "\n";
    print "data messages    = ", ($data_msgs/10), "\n\n\n";
}


print "----------TESTE TESTE TESTE--------\n\n";

($control_msgs, $data_msgs) = &processAODV("reboot-0-30-0.2-16780-7120-3072-60.0-512-10.0-50.0");

print "control messages = ", $control_msgs, "\n";
print "data messages    = ", $data_msgs, "\n\n\n";

($control_msgs, $data_msgs) = &processAODV("reboot-5-30-0.2-16780-29007-3072-60.0-512-10.0-50.0");

print "control messages = ", $control_msgs, "\n";
print "data messages    = ", $data_msgs, "\n\n\n";

($control_msgs, $data_msgs) = &processAODV("reboot-9-30-0.2-16780-6213-3072-60.0-512-10.0-50.0");

print "control messages = ", $control_msgs, "\n";
print "data messages    = ", $data_msgs, "\n\n\n";

#------ END OF MAIN PROGRAM -----------------------------------------

#------ SUBROUTINE computeStats -------------------------------------

sub computeStats {

    # input: a list with a collection of samples

    local(@samples) = $_[0];
    local($sum) = 0;
    local($s);

    foreach $s (@samples) {
	$sum += $s;
    }

    return $sum/($#samples+1);
}

#------ SUBROUTINE processAODV --------------------------------------

sub processAODV {

    # input: filename

    local($f) = $_[0]; 
    local($control) = 0;
    local($data) = 0;
    local($type);
    local($number);
        
    open(EXPFILE, "<$f");
    print "processing $f\n";

    while (<EXPFILE>) {
	
	if (/\s+aodv\s+number of\s+(.+)\s+([0-9]+)$/) {
	    $type = $1;
	    $number = $2;
	    if (($type =~ /data sent/) || ($type =~ /data forwarded/)) {
		$data += $number;
	    } elsif (!(($type =~ /boot/) || ($type =~ /pkts/) || 
		       ($type =~ /link/) || ($type =~ /abort/))) {
		$control += $number;
	    }
	}	           
    }
    
    close(EXPFILE);

    print "results= $control, $data\n\n";
    return ($control, $data);
}

#--------------------------------------------------------------------
