#
# Copyright (c) 2012 Bucknell University
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation;
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Author: L. Felipe Perrone (perrone@bucknell.edu)
#
#!/usr/bin/perl

if ($#ARGV < 2) {
    die("Usage: genplots.pl [parameters file] [experiment id]");
}

open(INPUT_FILE, $ARGV[0]) ||
    die("Error: Cannot open parameters file for reading.");

$parms_file = $ARGV[0];
$expId = $ARGV[1];

$expName = "";
$numRuns = 0;
$runId = 0;

while ($line = <INPUT_FILE>) {

    chop($line);

    @parsed_line = split(/\s+/,$line);

    if ($parsed_line[0] eq "SWANPATH") {
	$swanPath = $parsed_line[1];
    } elsif ($parsed_line[0] eq "EXPNAME") {
	$expName = $parsed_line[1];
    } elsif ($parsed_line[0] eq "NUMRUNS") {
	$numRuns_start = $parsed_line[1];
	$numRuns_end = $parsed_line[2];
    } elsif ($parsed_line[0] eq "SEED") {
	srand($parsed_line[1]);
    } elsif ($parsed_line[0] eq "NUMNODES") {
	for ($i=1; $i<=$#parsed_line; $i++) {
	    push(@num_nodes, $parsed_line[$i]);
	}
    } elsif ($parsed_line[0] eq "ATTACKPROB") {
	for ($i=1; $i<=$#parsed_line; $i++) {
	    push(@attack_prob, $parsed_line[$i]);
	}
    } elsif ($parsed_line[0] eq "PACKETSIZE") {
	for ($i=1; $i<=$#parsed_line; $i++) {
	    push(@packet_size, $parsed_line[$i]);
	}
    } elsif ($parsed_line[0] eq "BITRATE") {
	for ($i=1; $i<=$#parsed_line; $i++) {
	    push(@bitrate, $parsed_line[$i]);
	}
    } elsif ($parsed_line[0] eq "SESSLEN") {
	for ($i=1; $i<=$#parsed_line; $i++) {
	    push(@session_length, $parsed_line[$i]);
	}
    } elsif ($parsed_line[0] eq "ONLENGTH") {
	for ($i=1; $i<=$#parsed_line; $i++) {
	    push(@on_length, $parsed_line[$i]);
	}
    } elsif ($parsed_line[0] eq "OFFLENGTH") {
	for ($i=1; $i<=$#parsed_line; $i++) {
	    push(@off_length, $parsed_line[$i]);
	}
    } elsif ($parsed_line[0] eq "ATTACKJITTER") {
	for ($i=1; $i<=$#parsed_line; $i++) {
	    push(@jitter, $parsed_line[$i]);
	}
    } elsif ($parsed_line[0] eq "RESULTDIR") {
	$dir = $parsed_line[1];
    } elsif ($parsed_line[0] eq "DMLOUT") {
	$outdml = $parsed_line[1];
    } elsif ($parsed_line[0] eq "DMLIN") {
	$indml = $parsed_line[1];
    }
}

# create root directory for results, if it doesn't exist
if (!opendir(DIRHANDLE1,"$dir")) {
    die ("directory $dir not found");
}
else {
    closedir(DIRHANDLE1);
}

# create sub-directory for named experiment & id, if it doesn't exist
$curr_dir = "$dir/$expName_$expId";
if (!opendir(DIRHANDLE1,"$curr_dir")) {
    die ("directory $curr_dir not found");
}
else {
    closedir(DIRHANDLE1);
}

foreach $b (@bitrate) {
 
    $sub_rate = "rate-$b";
    $curr_dir = "$curr_dir/$sub_rate";

    # create sub-directory for bit rate
    if (!opendir(DIRHANDLE1,"$curr_dir")) {
	die ("directory $curr_dir not found");
    }
    else {
	closedir(DIRHANDLE1);
    }

    foreach $ps (@packet_size) {

	$sub_psize = "psize-$ps";
	$curr_dir = "$curr_dir/$sub_psize";
	
	# create sub-directory for packet size
	if (!opendir(DIRHANDLE1,"$curr_dir")) {
	    die ("directory $curr_dir not found");
	}
	else {
	    closedir(DIRHANDLE1);
	}
	
	foreach $sl (@session_length) {

	    $sub_slen = "slen-$sl";
	    $curr_dir = "$curr_dir/$sub_slen";
	    
	    # create sub-directory for session length
	    if (!opendir(DIRHANDLE1,"$curr_dir")) {
		die ("directory $curr_dir not found");
	    }
	    else {
		closedir(DIRHANDLE1);
	    }	 
   		    
	    for ($len=0; $len <= $#off_length; $len++) {
		$onl = $on_length[$len];
		$offl = $off_length[$len];
		
		foreach $n (@num_nodes) {
		    
		    foreach $j (@jitter) {
			
			foreach $p (@attack_prob) {
			    
			    $sub1 = "on-$onl-off-$offl";
			    $sub2 = "$n-jitter-$j";

			    if (!opendir(DIRHANDLE1,"$curr_dir/$sub1/$sub2")) {
				die ("directory $curr_dir/$sub1 not found");
								
				if (!opendir(DIRHANDLE2,"$curr_dir/$sub1/$sub2")) {
				    die("directory $curr_dir/$sub1/$sub2 not found");
				}
				else {
				    closedir(DIRHANDLE2);
				}
			    } 
			    else {
				closedir(DIRHANDLE1);
			    }

			    print "current directory $curr_dir/$sub1/$sub2\n";
			}
		    }
		}
	    }
	}
    }
}

close(EXPGROUPS);


