#!/usr/bin/env python
# import argparse
from optparse import OptionParser
from safe.boolean import *
import experiment
import logging
import handlers
import sys

class ExpCompiler(object):

    def __init__(self, xml, output_filename):
        self.xml = xml
        self.output_filename = output_filename
        self.exp = experiment.CreateFromDocument(self.xml)

    def compile(self):
        for member_of_constraint in self.exp.conditions.memberof:
            factor = member_of_constraint.factor
            list_identifier = member_of_constraint.listid
            values = []
            found_list = False

            # Iterate through all of the list elements in the experiment
            # space.  If one of them has an 'id' attribute which matches
            # our list_identifier, use that.  Otherwise, assume the
            # experimenter was specifying an external file.
            for levellist in self.exp.conditions.levellist:
                if levellist.id == list_identifier:
                    found_list = True
                    # We've found the right list.  Break out of loop.
                    break

            if not(found_list):
                # Assume they had the .xml extension already on there
                filename = str(list_identifier)
                xml = file(filename).read()
                l = experiment.CreateFromDocument(xml)
                ldom = l.toDOM()
                dom = self.exp.toDOM()
                conditions = dom.getElementsByTagName('ns1:conditions')[0]
                oldChild = conditions.getElementsByTagName('ns1:levellist')[0]
                newChild = ldom.getElementsByTagName('ns1:levellist')[0]
                newChild.setAttribute('id', list_identifier)
                conditions.insertBefore(newChild, oldChild)
                f = open(self.output_filename, 'w')
                f.write(dom.toprettyxml())
                f.close()

if __name__ == '__main__':

    usage = './%prog EXPERIMENT_DESCRIPTION.xml [options]'
    parser = OptionParser(usage = usage)
    parser.add_option('-v', '--verbose', dest='verbose',
                        action='store_true', default=False,
                        help='print verbose output')
    (options, args) = parser.parse_args()


    logger = logging.getLogger('expparser_logger')
    logger.setLevel(logging.DEBUG)

    if (options.verbose):
        ch = logging.StreamHandler()
        ch.setLevel(logging.DEBUG)
        logger.addHandler(ch)
    else:
        nh = handlers.NullHandler()
        nh.setLevel(logging.DEBUG)
        logger.addHandler(nh)

    xml_string = file(args[0]).read()
    output_filename = args[1]
    p = ExpCompiler(xml_string, output_filename)
    p.compile()
