#!/usr/bin/env python
# import argparse
from optparse import OptionParser
from safe.boolean import *
from safe.exceptions import *
import experiment
import logging
import handlers
import sys

class ExpParser(object):

    def __init__(self, xml):
        self.xml = xml
        self.exp = experiment.CreateFromDocument(self.xml)
        self.mapped_values = {}
        self.restriction_expression = None
        self.MULT = 'MULT'
        self.FDIV = 'FDIV'
        self.IDIV = 'IDIV'
        self.PLUS = 'PLUS'
        self.MINUS = 'MINUS'
        self.MOD = 'MOD'
        self.POW = 'POW'
        self.allowed_tests = ['EQUALS', 'LT', 'GT']
        self.expression_parts = \
        ['lconst', 'rconst', 'lvar', 'rvar', 'lexpr', 'rexpr', 'op']
        self.oneside_expression_parts = \
        ['const', 'var', 'expr']

    # not sure if this is needed
    def parse_simple_expression(self, expr1, op, expr2):
        if op == self.PLUS:
            return float(expr1) + float(expr2)
        elif op == MINUS:
            return float(expr1) - float(expr2)
        elif op == self.MULT:
            return float(expr1) * float(expr2)
        elif op == DIV:
            return float(expr1) / float(expr2)
        elif op == MOD:
            return float(expr1) % float(expr2)

    def evaluate_expression(self, funcstring, where_clause):
        image = []
        lo = where_clause.range.lo
        hi = where_clause.range.hi
        delta = where_clause.range.delta
        domain = range(int(lo), int(hi) + 1, int(delta))
        f = lambda x: eval(funcstring)
        for x in domain:
            image.append(f(x))
        return image


    def parse_expression(self, argdict):
        if str(argdict['op']) == self.PLUS:
            op = '+'
        elif str(argdict['op']) == self.MINUS:
            op = '-'
        elif str(argdict['op']) == self.MULT:
            op = '*'
        elif str(argdict['op']) == self.FDIV:
            op = '/'
        elif str(argdict['op']) == self.IDIV:
            op = '/'
        elif str(argdict['op']) == self.MOD:
            op = '%'
        elif str(argdict['op']) == self.POW:
            op = '**'

        lconst = str(argdict['lconst'])
        rconst = str(argdict['rconst'])
        lvar = str(argdict['lvar'])
        rvar = str(argdict['rvar'])
        funcstring = ''


        if not(argdict['lconst'] == None) and not(argdict['rconst'] == None):
            funcstring = '%s %s %s' % (lconst, op, rconst)
        elif not(argdict['lconst'] == None) and not(argdict['rvar'] == None):
            funcstring = '%s %s %s' % (lconst, op, 'x')
        elif not(argdict['lconst'] == None) and not(argdict['rexpr'] == None):
            newargdict = {}
            for part in self.expression_parts:
                p = eval('argdict[\'rexpr\'].%s' % part)
                newargdict[part] = p
            newfuncstring = self.parse_expression(newargdict)
            funcstring = '%s %s (%s)' % (lconst, op, newfuncstring)

        elif not(argdict['lvar'] == None) and not(argdict['rconst'] == None):
            funcstring = '%s %s %s' % ('x', op, rconst)
        elif not(argdict['lvar'] == None) and not(argdict['rvar'] == None):
            funcstring = '%s %s %s' % ('x', op, 'x')
        elif not(argdict['lvar'] == None) and not(argdict['rexpr'] == None):
            newargdict = {}
            for part in self.expression_parts:
                p = eval('argdict[\'rexpr\'].%s' % part)
                newargdict[part] = p
            newfuncstring = self.parse_expression(newargdict)
            funcstring = '%s %s (%s)' % ('x', op, newfuncstring)

        elif not(argdict['lexpr'] == None) and not(argdict['rconst'] == None):
            newargdict = {}
            for part in self.expression_parts:
                p = eval('argdict[\'lexpr\'].%s' % part)
                newargdict[part] = p
            newfuncstring = self.parse_expression(newargdict)
            funcstring = '(%s) %s %s' % (newfuncstring, op, rconst)
        elif not(argdict['lexpr'] == None) and not(argdict['rvar'] == None):
            newargdict = {}
            for part in self.expression_parts:
                p = eval('argdict[\'lexpr\'].%s' % part)
                newargdict[part] = p
            newfuncstring = self.parse_expression(newargdict)
            funcstring = '(%s) %s %s' % (newfuncstring, op, 'x')
        elif not(argdict['lexpr'] == None) and not(argdict['rexpr'] == None):
            lnewargdict = {}
            for part in self.expression_parts:
                p = eval('argdict[\'lexpr\'].%s' % part)
                lnewargdict[part] = p

            rnewargdict = {}
            for part in self.expression_parts:
                p = eval('argdict[\'rexpr\'].%s' % part)
                rnewargdict[part] = p
            lfuncstring = self.parse_expression(lnewargdict)
            rfuncstring = self.parse_expression(rnewargdict)
            funcstring = '(%s) %s (%s)' % (lfuncstring, op, rfuncstring)
        return funcstring

    def apply_test(self, factor, test, value):
        def equals(x): return float(x) == float(value)
        def leq(x):    return float(x) <= float(value)
        def geq(x):    return float(x) >= float(value)

        if test == 'EQUALS':
            keep_list = filter(equals, self.mapped_values[factor])
        elif test == 'LEQ':
            keep_list = filter(leq, self.mapped_values[factor])
        elif test == 'GEQ':
            keep_list = filter(geq, self.mapped_values[factor])

        return keep_list

    def parse_sequence(self):
        # logger.debug('Parsing condition: Value-Constraint')
        # logger.debug('-----------------------------------')
        for sequence in self.exp.conditions.sequence:
            factor = sequence.factor
            if not(str(factor) in self.mapped_values.keys()):
                e = UndeclaredFactorException()
                e.factor_name = str(factor)
                e.msg =  'Factor name \'%s\' in <memberof> element '
                e.msg += 'does not appear in <factorlist>'
                e.msg = e.msg % str(factor)
                raise e
            test = sequence.test
            if not(str(test) in self.allowed_tests):
                e = InvalidTestStringException()
                e.test_string = str(test)
                e.msg =  'Test \'%s\' is not a valid test. '
                e.msg += 'Must be one of %s'
                e.msg = e.msg % (str(test), self.allowed_tests)
                raise e
            argdict = {}
            # left-side constant, variable, expression
            argdict['lexpr'] = sequence.lexpr
            argdict['lvar'] = sequence.lvar
            argdict['lconst'] = sequence.lconst
            # right-side constant, variable, expression
            argdict['rexpr'] = sequence.rexpr
            argdict['rvar'] = sequence.rvar
            argdict['rconst'] = sequence.rconst
            argdict['op'] = sequence.op
            
            where_clause = sequence.where

            # Parsing condition: onTime = i + 10 where 0 <= i <= 10
            # logger.info('Parsing condition: %s %s %s %s %s' %\
            # (str(factor), str(test), str(expr1), str(op), str(expr2)))

            if where_clause != None:
                funcstring = self.parse_expression(argdict)
                # logger.debug('%s = %s where %s <= x <= %s' %\
                # (str(factor), funcstring, str(where_clause.range.lo), str(where_clause.range.hi)))
                keep_list = self.evaluate_expression(funcstring, where_clause)
            else:
                pass
                # TODO: handle this.
                # value = parseSimpleExpression(expr1, op, expr2)
                # keep_list = applyTest(factor, test, value)

            self.mapped_values[str(factor)].extend(keep_list)

    def parse_member_of(self):
        for member_of_constraint in self.exp.conditions.memberof:
            factor = member_of_constraint.factor
            if not(str(factor) in self.mapped_values.keys()):
                e = UndeclaredFactorException()
                e.factor_name = str(factor)
                e.msg =  'Factor name \'%s\' in <memberof> element '
                e.msg += 'does not appear in <factorlist>'
                e.msg = e.msg % str(factor)
                raise e

            list_identifier = member_of_constraint.listid
            values = []
            found_list = False

            # Iterate through all of the list elements in the experiment
            # space.  If one of them has an 'id' attribute which matches
            # our list_identifier, use that.  Otherwise, assume the
            # experimenter was specifying an external file.
            # TODO: Make this more explicit in the experiment description.
            for levellist in self.exp.conditions.levellist:
                if levellist.id == list_identifier:
                    values = map(str, levellist.level)
                    found_list = True
                    # We've found the right list.  Break out of loop.
                    break

            if not(found_list):
                # Assume they had the .xml extension already on there
                try:
                    filename = str(list_identifier)
                    xml = file(filename).read()
                    levellist = experiment.CreateFromDocument(xml)
                    values = map(str, levellist.level)
                except IOError:
                    # TODO: something better than this hack.
                    e = MissingListException()
                    e.list_id= str(list_identifier)
                    e.msg =  'List identifier \'%s\' does not appear as '
                    e.msg += 'the @id attribute of any <levellist> '
                    e.msg += 'nor the filename of any external file.'
                    e.msg = e.msg % str(list_identifier)
                    raise e
                    

            # Assert: at this point, values will be something like
            # values = ['0', '1', '10', '14'].  Turn these into floats.
            keep_list = map(float, values)
            self.mapped_values[str(factor)].extend(keep_list)

    def parse_exclusion_restriction(self):
        """Add the exclusion combinations we don't care about to the list of
        throwaway points."""
        or_expression = None

        for restriction in self.exp.conditions.exclusionrestriction:
            point = {}
            for setting in restriction.setting:
                point[str(setting.factor)] = float(setting.level)

            # This says an INVALID design point is one that contains
            # {'A':2, 'B':5} OR {'A':5, 'B':9} OR ...
            if or_expression == None:
                or_expression = AndList(point)
            else:
                or_expression = Expression(or_expression, 'or', AndList(point))

        # This says a valid design point is
        # NOT ({'A':2, 'B':5} OR {'A':5, 'B':9} OR ...)
        if self.restriction_expression == None and or_expression == None:
            self.restriction_expression = None
        elif self.restriction_expression == None and or_expression != None:
            self.restriction_expression = -or_expression
        elif self.restriction_expression != None and or_expression != None:
            self.restriction_expression = Expression(self.retriction_expression, 'and', -or_expression)
        else:
            pass

    def parse_linking_restriction(self):
        """For the factors given, any levels which do not appear at the
        same index in their list are considered to be throwaway points."""


        and_expression = None
        for restriction in self.exp.conditions.linkingrestriction:
            minimum_size = 0
            or_expression = None

            # We're getting the size of the smallest list of factors
            for factor in restriction.factor:

                # Check if this is a valid factor and raise an exception if
                # not.
                if not(str(factor) in self.mapped_values.keys()):
                    e = UndeclaredFactorException()
                    e.factor_name = str(factor)
                    e.msg =  'Factor name \'%s\' in <linkingrestriction> element '
                    e.msg += 'does not appear in <factorlist>'
                    e.msg = e.msg % str(factor)
                    raise e

                if minimum_size == 0:
                    minimum_size = len(self.mapped_values[str(factor)])
                elif len(self.mapped_values[str(factor)]) < minimum_size:
                    minimum_size = len(self.mapped_values[str(factor)])

            # Create a design point corresponding to each possible index
            # number.  I.e., if we have
            #
            # ONTIME = [1, 2, 3, 4]
            # OFFTIME = [12, 25, 92, 7]
            #
            # Then create p1 = {'ONTIME' : 1, 'OFFTIME' : 12}
            #             p2 = {'ONTIME' : 2, 'OFFTIME' : 25}
            #             p3 = {'ONTIME' : 3, 'OFFTIME' : 92}
            #             p4 = {'ONTIME' : 4, 'OFFTIME' :  7}
            for i in range(0, minimum_size):
                point = {}
                for factor in restriction.factor:
                    fact = str(factor)
                    point[fact] = self.mapped_values[fact][i]

                # This says a valid design point must contain
                # {'A':0, 'B':0} OR {'A':1, 'B':1} OR {'A':2, 'B':2} OR ...
                if or_expression == None:
                    or_expression = AndList(point)
                else:
                    or_expression = Expression(AndList(point), 'or', or_expression)

            # This says a valid design point must contain
            # {'A':0, 'B':0} OR {'A':1, 'B':1} OR {'A':2, 'B':2} OR ...
            #     AND
            # {'C':0, 'D':0} OR {'C':1, 'D':1} OR {'C':2, 'D':2} OR ...
            #     AND
            # {'E':0, 'F':0} OR {'E':1, 'F':1} OR {'E':2, 'F':2} OR ...
            if and_expression == None:
                and_expression = or_expression
            else:
                and_expression = Expression(and_expression, 'and', or_expression)

        # Append this linking restriction to the (perhaps)
        # already-existing self.restriction_expression.
        if self.restriction_expression == None and and_expression == None:
            self.restriction_expression = None
        elif self.restriction_expression == None and and_expression != None:
            self.restriction_expression = and_expression
        elif self.restriction_expression != None and and_expression != None:
            self.restriction_expression = Expression(self.restriction_expression, 'and', and_expression)
        else:
            pass


    def get_restriction_expression(self):
        """Get the safe.boolean.Expression representation of this
        experiment's restrictions."""

        return self.restriction_expression

    def get_factors_levels(self):
        """Get the factor-levels dictionary."""

        return self.mapped_values

    def get_exclusive_restrictions(self):
        """Get the experiment's exclusivity-based restrictions."""

        return self.excluded_points

    def get_inclusive_restrictions(self):
        """Get the experiment's inclusive-based restrictions."""

        return self.included_points

    def parse(self):
        for factor in self.exp.factorlist.factor:
            self.mapped_values[str(factor)] = []
        self.parse_member_of()
        self.parse_sequence()
        self.parse_exclusion_restriction()
        self.parse_linking_restriction()

    def print_mapped_values(self):
        for factor, levels in self.mapped_values.iteritems():
            print('%-12s %s' % (factor, levels))

if __name__ == '__main__':

    usage = './%prog EXPERIMENT_DESCRIPTION.xml [options]'
    parser = OptionParser(usage = usage)
    parser.add_option('-v', '--verbose', dest='verbose',
                        action='store_true', default=False,
                        help='print verbose output')
    (options, args) = parser.parse_args()


    logger = logging.getLogger('expparser_logger')
    logger.setLevel(logging.DEBUG)

    if (options.verbose):
        ch = logging.StreamHandler()
        ch.setLevel(logging.DEBUG)
        logger.addHandler(ch)
    else:
        nh = handlers.NullHandler()
        nh.setLevel(logging.DEBUG)
        logger.addHandler(nh)

    xml_string = file(args[0]).read()
    p = ExpParser(xml_string)
    p.parse()
    print('Experiment Space (factors and levels)')
    print('-------------------------------------')
    p.print_mapped_values()
