#!/usr/bin/env python
## -*- Mode: python; py-indent-offset: 4; indent-tabs-mode: nil; coding: utf-8; -*-
#
# Copyright (c) 2013 Bucknell University
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation;
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Author: William Stratton (wss012@bucknell.edu)
#

import requests
import argparse
import os
import json
import common
import sys

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('--installation', '-i',
            required=True,
            help="installation name"),
    parser.add_argument('--owner', '-o',
            default = None,
            help="owner of the installation"),
    parser.add_argument('--file', '-f',
            required=True,
            help="path to SLED file")

    args = parser.parse_args()

    with open(os.path.abspath(args.file)) as f:
        decoder = json.JSONDecoder()
        d = decoder.decode(f.read())
        d['installation'] = args.installation
        d['owner'] = args.owner

    config = common.get_config()
    header = {}
    header['Authorization'] = 'Token ' + config['token']

    header['content-type'] = 'application/json'
    header['Accept'] = 'application/json'

    r = requests.post(config['base-url'] + "installations/SLED/", data = json.dumps(d), headers=header)

    if r.status_code >= 400:
        sys.stderr.write("Error creating experiment description\n")
        return 1
    print("Success")
    return 0

if __name__ == "__main__":
    exit_code = main()
    if exit_code != 0:
        sys.exit(exit_code)

