#!/usr/bin/env python
## -*- Mode: python; py-indent-offset: 4; indent-tabs-mode: nil; coding: utf-8; -*-
#
# Copyright (c) 2013 Bucknell University
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation;
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Author: Nicholas Marshall (nam021@bucknell.edu) 
# based off safe_crete_user by William Stratton (wss012@bucknell.edu)
#

import argparse
import requests
import getpass
import json
import common
import re
import sys

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('--path', '-p',
            required=True,
            default=None,
            help="Path to worker directory")
    parser.add_argument('--host', '-s',
            required=True,
            default=None,
            help="Hostname of worker")
    parser.add_argument('--children', '-c',
            required=False,
            default="1",
            help="Number of child processes the worker can spawn")
    args = parser.parse_args()


    # Get config information and set header
    config = common.get_config()
    header = {'Authorization':  'Token ' + config['token']}
    header['content-type'] = 'application/json'

    data = {'path':args.path, 'host':args.host, 'connected':'0', 'children':args.children}

    r = requests.post(config['base-url']+'workers/', headers = header, data=json.dumps(data))

    if r.status_code >= 400:
        print("Error creating Worker")
        return 1
    else:
        print("Success")
        return 0
    
if __name__ == "__main__":
    if main() != 0:
        sys.exit(1)


