#!/usr/bin/env python
## -*- Mode: python; py-indent-offset: 4; indent-tabs-mode: nil; coding: utf-8; -*-
#
# Copyright (c) 2013 Bucknell University
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation;
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Author: William Stratton (willstrat@gmail.com)
#

"""
Fills the database with some data that can be used to test the
visualization software. It then prints the id of the simulation
run that contains the data.
"""

import os
os.environ.setdefault("DJANGO_SETTINGS_MODULE", "config.settings")

from safe.models import *
import datetime


o = User(username="Test User", password='password')
o.save()

i = Installation(name="Test Installation", description="An installation for testing. Does not include ns-3", tarball_path=os.path.abspath(__file__), simulator_folder=" ", owner=o, public=1)
i.save()

d = ExpDescription(name="Test Experiment Description", script="script name", installation=i, owner=o, public=1, description="An experiment description for testing")
d.save()

e = Experiment(name="Test Experiment", description="An experiment for testing", replications=1, exp_description=d, owner=o, public=1, create_date = datetime.datetime.now())
e.save()

dpt = DesignPoint(experiment = e)
dpt.save()

w = Worker(path="a path should go here", host="and a host name here", auth_key="and authentication info here", connected='0')
w.save()

s = SimulationRun(design_point=dpt, start_time=datetime.datetime.now(), end_time=datetime.datetime.now(), isFinished=1, worker=w, success=1)
s.save()
print('The simulation run id is '+str(s.id))

m = Metric(name='First Metric', exp_description=d, data_type='double', units='spleens')
m.save()

r = ResultSet(metric=m, simulation_run=s)
r.save()

data = DoubleData(type='R', code=r.id, time=1, value=0.7)
data.save()

data = DoubleData(type='R', code=r.id, time=2, value=2.5)
data.save()

m = Metric(name='Second Metric', exp_description=d, data_type='double', units='livers')
m.save()

r = ResultSet(metric=m, simulation_run=s)
r.save()

data = DoubleData(type='R', code=r.id, time=0, value=0.2)
data.save()

data = DoubleData(type='R', code=r.id, time=2, value=3.1)
data.save()

m = Metric(name='Third Metric', exp_description=d, data_type='double', units='kidneys')
m.save()

r = ResultSet(metric=m, simulation_run=s)
r.save()

data = DoubleData(type='R', code=r.id, time=1, value=5.4)
data.save()

data = DoubleData(type='R', code=r.id, time=3, value=1.5)
data.save()
