//--------------------------------
// Base Magnitude
// -------------------------------

function BaseMagnitude(){
}

var baseMagnitudeNumber = 0;
BaseMagnitude.prototype.init = function(margins, width, height, selectedCallBack, parentDomId){
    this.xMin = this.yMin = this.xMax = this.yMax = 0;
    this.data = {};
    this.toDisplay = {};
    this.pointAttributes = {};
    this.mouseEvents = {};
    this.jMargin = margins;
    this.height = height - this.jMargin.top - this.jMargin.bottom;
    this.width = width - this.jMargin.left - this.jMargin.right;
    this.selectedCallBack = selectedCallBack;
    this.id = "base_magnitude_" + baseMagnitudeNumber;
    baseMagnitudeNumber += 1;
    console.log(this, parentDomId);

    this.dXScale = d3.scale.linear().range([0, this.width]).domain([0, 0]);
    this.dYScale = d3.scale.linear().range([this.height, 0]).domain([0, 0]);

    this.dXAxis = d3.svg.axis().scale(this.dXScale).orient("bottom");
    this.dYAxis = d3.svg.axis().scale(this.dYScale).orient("left");

    this.svg = d3.select("#" + parentDomId)
        .append("svg")
        .attr("width", this.width + this.jMargin.left + this.jMargin.right)
        .attr("height", this.height + this.jMargin.top + this.jMargin.bottom)
        .on("click", this.selected.bind(this));
    console.log("SVG:", this.svg);

	// Text label for x axis
	this.xAxisLabel = this.svg.append("text")
        .attr("class", "x label")
		.attr("x", this.width / 2)
		.attr("y", this.height + this.jMargin.top)
		.attr("dy", "3em")
		.style("text-anchor", "middle")
		.text("Original x-axis");

	// Add text label for y axis
	this.yAxisLabel = this.svg.append("text")
		.attr("class", "y label")
		.attr("transform", "rotate(-90)")
		.attr("y", 0)
		.attr("x", 0 - ((this.height) / 2))
		.attr("dy", "1em")
		.style("text-anchor", "middle")
		.text("Original y-axis");

	// Add x axis
	this.svg.append("g")
		.attr("class", "x axis")
		.attr("transform", "translate(" + this.jMargin.left + "," + (this.height + this.jMargin.top) + ")")
        .on("click", this.selected.bind(this))
		.call(this.dXAxis);

    // Add y axis
    this.svg.append("g")
		.attr("class", "y axis")
		.attr("transform", "translate(" + this.jMargin.left + "," + this.jMargin.top + ")")
		.call(this.dYAxis);

    // Set the cliping area
    this.clip = this.svg.append("defs").append("clipPath")
        .attr("id", this.id + "_clip")
        .append("rect")
        .attr("id", this.id + "_clip-rect")
        .attr("y", this.jMargin.top)
        .attr("x", this.jMargin.left)
        .attr("width", this.width)
        .attr("height", this.height);

    return this;
}

BaseMagnitude.prototype.selected = function(){
    if(this.selectedCallBack){ this.selectedCallBack(this) }
}

BaseMagnitude.prototype.setXAxisLabel = function(label){
    this.xAxisString = label;
    this.svg.selectAll(".x.label").text(label);
}

BaseMagnitude.prototype.setYAxisLabel = function(label){
    this.yAxisString = label;
    this.svg.selectAll(".y.label").text(label);
}

BaseMagnitude.prototype.getXAxisLabel = function(){
    return this.xAxisString;
}

BaseMagnitude.prototype.getYAxisLabel = function(){
    return this.yAxisString;
}

BaseMagnitude.prototype.getWidth = function(){
    return this.width + this.jMargin.left + this.jMargin.right;
}

BaseMagnitude.prototype.getHeight = function(){
    return this.height + this.jMargin.top + this.jMargin.bottom;
}

BaseMagnitude.prototype.displayAll = function(){
    Object.keys(this.data).forEach(function(id){
        if(this.toDisplay[id] == true) { this.display(id) }
    }.bind(this));
}

BaseMagnitude.prototype.addDataSet = function(id, data, pointAttributes, mouseEvents){
    this.data[id] = data;
    this.pointAttributes[id] = pointAttributes;
    this.mouseEvents[id] = mouseEvents;
    this.toDisplay[id] = true;
    this.updateDataFeatures();
}

BaseMagnitude.prototype.getDataSet = function(id){
    return this.data[id];
}

BaseMagnitude.prototype.getPointAttributes = function(id){
    return this.pointAttributes[id];
}

BaseMagnitude.prototype.getMouseEvents = function(id){
    return this.mouseEvents[id];
}

BaseMagnitude.prototype.setInvisible = function(id){
    this.toDisplay[id] = false;
    this.updateDataFeatures();
    this.display(id);
}

BaseMagnitude.prototype.setVisible = function(id){
    this.toDisplay[id] = true;
    this.updateDataFeatures();
    this.display(id);
}

BaseMagnitude.prototype.getVisibleIds = function(){
    values = [];
    Object.keys(this.toDisplay).forEach(function(id){
        if(this.toDisplay[id] == true)
            values.push(id);
    }.bind(this));
    return values;
}

BaseMagnitude.prototype.remove = function(id){
    delete this.data[id];
    delete this.pointAttributes[id];
    delete this.mouseEvents[id];
}

BaseMagnitude.prototype.removeAll = function(){
    Object.keys(this.data).forEach(function(id){
        this.setInvisible(id);
        this.remove(id);
    }.bind(this));
}

BaseMagnitude.prototype.hasData = function(id){
    if( this.data[id] != undefined ){ return true }
    else { return false }
}

BaseMagnitude.prototype.setHeight = function(height){
    this.setSize(this.getWidth(), height);
}

BaseMagnitude.prototype.setWidth = function(width){
    this.setSize(width, this.getHeight());
}

BaseMagnitude.prototype.setSize = function(width, height){
    this.svg
        .attr("width", width)
        .attr("height", height);

    this.width = width - this.jMargin.left - this.jMargin.right;
    this.height = height - this.jMargin.top - this.jMargin.bottom;

	this.xAxisLabel.attr({"x": this.width / 2, "y": this.height + this.jMargin.top } );
	this.yAxisLabel.attr("x", 0-((this.height) / 2));


	this.svg.select(".x.axis").attr("transform", "translate(" + this.jMargin.left + "," + (this.height + this.jMargin.top) + ")");

    this.dYScale.range([this.height, 0]);
    this.dXScale.range([0, this.width]);

    this.svg.select(".x.axis").call(this.dXAxis);
    this.svg.select(".y.axis").call(this.dYAxis);

    d3.select("#" + this.id + "_clip-rect")
        .attr("x", this.jMargin.left)
        .attr("y", this.jMargin.top)
        .attr("width", this.width)
        .attr("height", this.height);
}

BaseMagnitude.prototype.updateDomains = function(xDomain, yDomain){
    if(xDomain != undefined) { this.dXScale.domain(xDomain) }
    if(yDomain != undefined) { this.dYScale.domain(yDomain) }

    this.svg.select(".x.axis").call(this.dXAxis);
    this.svg.select(".y.axis").call(this.dYAxis);
}

BaseMagnitude.prototype.updateDataFeatures = function(){
    this.xMin = this.xMax = this.yMin = this.yMax = undefined;
    Object.keys(this.data).forEach(function(id){
        if(this.toDisplay[id] != true){ return }

        xRange = d3.extent(this.data[id], function(d) { return d.time });
        this.xMin = (this.xMin == undefined) ? xRange[0] : d3.min([this.xMin, xRange[0]]);
        this.xMax = (this.xMax == undefined) ? xRange[1] : d3.max([this.xMax, xRange[1]]);

        yRange = d3.extent(this.data[id], function(d) { return d.value});
        this.yMin = (this.yMin == undefined) ? yRange[0] : d3.min([this.yMin, yRange[0]]);
        this.yMax = (this.yMax == undefined) ? yRange[1] : d3.max([this.yMax, yRange[1]]);
        }.bind(this));

    console.log(this.xMin, this.xMax);
    this.updateDomains([this.xMin, this.xMax], [this.yMin, this.yMax]);
}

BaseMagnitude.prototype.updateCrossHairs = function(){
    var x = d3.event.offsetX;
    var y = d3.event.offsetY;

    var xleft = this.dXScale.range()[0] + this.jMargin.left; 
    var xright = this.dXScale.range()[1] + this.jMargin.left;
    var ytop = this.dYScale.range()[0] + this.jMargin.top;
    var ybottom = this.dYScale.range()[1] + this.jMargin.top;

    if (x < xleft) { x = xleft }
    if (x > xright) { x = xright }
    if (y < ybottom) { y = ybottom }
    if (y > ytop) { y = ytop }

    horizontal = this.svg.selectAll(".crosshairhorizontal").data([""]);
    vertical = this.svg.selectAll(".crosshairvertical").data([""]);
 
    horizontal.enter().append("line").attr("class", "crosshairhorizontal").style("stroke", "black").style("pointer-events", "none")
    vertical.enter().append("line").attr("class", "crosshairvertical").style("stroke", "black").style("pointer-events", "none")
   
    horizontal.attr("x1", xleft).attr("x2", xright).attr("y1", y).attr("y2", y);
    vertical.attr("y1", ybottom).attr("y2", ytop).attr("x1", x).attr("x2", x);

    horizontal.exit().remove();
    vertical.exit().remove();
}

BaseMagnitude.prototype.enableCrossHairs = function(){
    this.svg.on("mousemove", this.updateCrossHairs.bind(this));
    this.svg.on("mouseout", function(){
        this.svg.selectAll(".crosshairhorizontal").remove();
        this.svg.selectAll(".crosshairvertical").remove();
        }.bind(this));
}

BaseMagnitude.prototype.disableCrossHairs = function(){
    this.svg.selectAll(".crosshairhorizontal").remove();
    this.svg.selectAll(".crosshairvertical").remove();
    this.svg.on("mousemove", undefined);

}

//--------------------------------
// Scatterplot Magnitude
// -------------------------------

ScatterMagnitude.prototype = new BaseMagnitude();
ScatterMagnitude.prototype.constructor = ScatterMagnitude;

function ScatterMagnitude(margins, width, height, onSelectedCallBack, parentDomId){
    BaseMagnitude.apply(this);
    this.init(margins, width, height, onSelectedCallBack, parentDomId); 

    return this;
}

ScatterMagnitude.prototype.display = function(id){
    if(this.toDisplay[id] != true) { 
        this.svg.selectAll(".dataPoint_" + id).remove();
        return;
    }

    //currentData = [];
    //this.data[id].forEach(function(dataPoint){
    //    if(this.dXScale.domain()[0] <= dataPoint.time && dataPoint.time <= this.dXScale.domain()[1])
    //        if(this.dYScale.domain()[0] <= dataPoint.value && dataPoint.value <= this.dYScale.domain()[1])
    //            currentData.push(dataPoint);
    //    }.bind(this));

    points = this.svg.selectAll(".dataPoint_" + id).data(this.data[id]);
    points.enter()
        .append("circle")
        .attr("clip-path", "url(#" + this.id + "_clip)" )
        .attr("class", "dataPoint_" + id)
        .on(this.mouseEvents[id]); 

    points
        .attr("cx", function (d) { return this.jMargin.left + this.dXScale(d.time) }.bind(this))
        .attr("cy", function (d) { return this.jMargin.top + this.dYScale(d.value) }.bind(this))
        .attr(this.pointAttributes[id]);

	points.exit().remove();	
}


//--------------------------------
// Lineplot Magnitude
// -------------------------------

LineMagnitude.prototype = new BaseMagnitude();
LineMagnitude.prototype.constructor = LineMagnitude;

function LineMagnitude(margins, width, height, onSelectedCallBack, parentDomId, pad){
    //console.log("Initializing LineMagnitude...");
    BaseMagnitude.apply(this);
    this.pad = (pad == undefined) ? false : pad;
    this.init(margins, width, height, onSelectedCallBack, parentDomId);

    this.line = d3.svg.line()
        .x(function (d) { return this.jMargin.left + this.dXScale(d["xValue"]) }.bind(this))
        .y(function (d) { return this.jMargin.top + this.dYScale(d["medianValue"]) }.bind(this));

    return this;
}

LineMagnitude.prototype.display = function(id){
    if(this.toDisplay[id] != true) { 
        //console.log("Removing...", this.toDisplay[id]);
        this.svg.selectAll(".dataGroup_" + id).remove();
        this.svg.selectAll(".dataPath_" + id).remove();
        return;
    }

    //============================================
    groups = this.svg.selectAll(".dataGroup_" + id).data(this.data[id]);
    groups.enter().append("g").attr("class", "dataGroup_" + id);
    groups.exit().remove();

    //points = groups.selectAll(".subPoint").data(["minValue", "lowDevValue", "medianValue", "highDevValue", "maxValue"]);
    points = groups.selectAll(".subPoint").data(function(d, i, f){ 
        values =
               [ {"cx": d["xValue"], "cy": d["lowDevValue"]},
                 {"cx": d["xValue"], "cy": d["medianValue"]},
                 {"cx": d["xValue"], "cy": d["highDevValue"]}] ;
        return values; });

    points.enter().append("circle").attr("class", "subPoint").on(this.mouseEvents[id]);
    points
        .attr("clip-path", "url(#" + this.id + "_clip)" )
        .attr("cx", function (d) { return this.jMargin.left + this.dXScale(d.cx) }.bind(this))
        .attr("cy", function (d) { return this.jMargin.top + this.dYScale(d.cy) }.bind(this))
        .attr(this.pointAttributes[id]);
    points.exit().remove();

    //============================================
 
    lines = groups.selectAll(".subline").data([""]);
    lines.enter().append("line").attr({"class":"subline", "stroke-width":"0.5", "stroke":this.pointAttributes[id].stroke});
    lines
        .attr("clip-path", "url(#" + this.id + "_clip)" )
        .attr("x1", function (d, e, f) { return this.jMargin.left + this.dXScale(this.data[id][f]["xValue"]); }.bind(this))
        .attr("x2", function (d, e, f) { return this.jMargin.left + this.dXScale(this.data[id][f]["xValue"]); }.bind(this))
        .attr("y1", function (d, e, f) { return this.jMargin.top + this.dYScale(this.data[id][f]["lowDevValue"]); }.bind(this))
        .attr("y2", function (d, e, f) { return this.jMargin.top + this.dYScale(this.data[id][f]["highDevValue"]); }.bind(this));
    lines.exit().remove();

    //=============================================

    paths = this.svg.selectAll(".dataPath_" + id).data([this.data[id]]);

    if (this.data[id].length > 1) {
        paths.enter()
            .append("path")
            .attr("class", "dataPath_" + id)
            .attr("stroke", this.pointAttributes[id].stroke)
            .attr("stroke-width", 1);

        paths
            .attr("clip-path", "url(#" + this.id + "_clip)" )
            .attr("d", function (d) { return this.line(d) }.bind(this));

        paths.exit().remove();
    }else{
        paths.remove();
    }
}

LineMagnitude.prototype.updateDataFeatures = function(){
    this.xMin = this.xMax = this.yMin = this.yMax = undefined;
    Object.keys(this.data).forEach(function(id){
        if(this.toDisplay[id] != true){ return }
        xRange = d3.extent(this.data[id], function(d) { return d["xValue"] });
        this.xMin = (this.xMin == undefined) ? xRange[0] : d3.min([this.xMin, xRange[0]]);
        this.xMax = (this.xMax == undefined) ? xRange[1] : d3.max([this.xMax, xRange[1]]);

        yMin = d3.min(this.data[id], function(d) { return d["lowDevValue"] });
        yMax = d3.max(this.data[id], function(d) { return d["highDevValue"] });

        yRange = d3.extent(this.data[id], function(d) { return d.value});
        this.yMin = (this.yMin == undefined) ? yMin : d3.min([this.yMin, yMin]);
        this.yMax = (this.yMax == undefined) ? yMax : d3.max([this.yMax, yMax]);
    }.bind(this));

    this.updateDomains([this.xMin, this.xMax], [this.yMin, this.yMax]);
}

LineMagnitude.prototype.updateDomains = function(xDomain, yDomain){
    var xValLeft, xValRight;
    range = xDomain[1] - xDomain[0];
    if(this.pad && range > 0){
        pad = 0.1 * range;
        xValLeft = xDomain[0] - pad;
        xValRight = xDomain[1] + pad;
    }
    else if (this.pad){
        pad = xDomain[1] > 10 ? 10 : this.xMax;
        xValLeft = xDomain[0] - pad;
        xValRight = xDomain[0] + pad;
    }else{
        xValLeft = xDomain[0];
        xValRight = xDomain[1];
    }
    

    //console.log("Range Used: ", xValLeft, xValRight);
	this.dXScale.range([0, this.width]).domain([xValLeft, xValRight]);
	this.dYScale.range([this.height, 0]).domain([0, this.yMax]);

	this.dYAxis.scale(this.dYScale);
	this.svg.select(".y.axis").call(this.dYAxis);

	this.dXAxis.scale(this.dXScale);
    this.svg.select(".x.axis").call(this.dXAxis);
}


//--------------------------------
// Box and Whiskers Magnitude
// -------------------------------

BoxAndWhiskersMagnitude.prototype = new BaseMagnitude();
BoxAndWhiskersMagnitude.prototype.constructor = BoxAndWhiskersMagnitude;

function BoxAndWhiskersMagnitude(margins, width, height, parentDomId, pad){
    //console.log("Initializing BoxAndWhiskersMagnitude...");
    BaseMagnitude.apply(this);
    this.pad = (pad == undefined) ? false : pad;
    this.init(margins, width, height, parentDomId);

    return this;
}

BoxAndWhiskersMagnitude.prototype.display = function(id){
    if(this.toDisplay[id] != true) { 
        //console.log("Removing...", this.toDisplay[id]);
        this.svg.selectAll(".dataGroup_" + id).remove();
        this.svg.selectAll(".dataPath_" + id).remove();
        return;
    }

    currentData = [];
	nextGroups = this.getNextGroups();
	dataNextLeft = nextGroups[0];
	datanextRight = nextGroups[1];
    

    //============================================

    groups = this.svg.selectAll(".dataGroup_" + id).data(currentData);
    groups.enter().append("g").attr("class", "dataGroup_" + id);
    groups.exit().remove();

    points = groups.selectAll(".subPoint").data(["minValue", "lowDevValue", "medianValue", "highDevValue", "maxValue"]);
    //points = groups.selectAll(".subPoint").data(["lowDevValue", "medianValue", "highDevValue"]);
    points.enter().append("circle").attr("class", "subPoint").on(this.mouseEvents[id]);
    points
        .attr("cx", function (d, e, f) { return this.jMargin.left + this.dXScale(currentData[f]["xValue"])}.bind(this))
        .attr("cy", function (d, e, f) { return this.jMargin.top + this.dYScale(currentData[f][d]) }.bind(this))
        .attr(this.pointAttributes[id]);
    points.exit().remove();

    //============================================
 
    lines = groups.selectAll(".subline").data([""]);
    lines.enter().append("line").attr({"class":"subline", "stroke-width":"0.5", "stroke-dasharray":"5,5", "stroke":this.pointAttributes[id].stroke});
    lines
        .attr("x1", function (d, e, f) { return this.jMargin.left + this.dXScale(currentData[f]["xValue"]); }.bind(this))
        .attr("x2", function (d, e, f) { return this.jMargin.left + this.dXScale(currentData[f]["xValue"]); }.bind(this))
        .attr("y1", function (d, e, f) { return this.jMargin.top + this.dYScale(currentData[f]["lowDevValue"]); }.bind(this))
        .attr("y2", function (d, e, f) { return this.jMargin.top + this.dYScale(currentData[f]["highDevValue"]); }.bind(this));
    lines.exit().remove();

    //=============================================
    line = d3.svg.line()
        .x(function (d) { return this.jMargin.left + this.dXScale(d["xValue"]) }.bind(this))
        .y(function (d) { return this.jMargin.top + this.dYScale(d["medianValue"]) }.bind(this));

    if (dataNextLeft.xValue != undefined) { currentData.unshift(dataNextLeft); }
    if (dataNextRight.xValue != undefined) { currentData.push(dataNextRight); }

    paths = this.svg.selectAll(".dataPath_" + id).data([""]);

    if (currentData.length > 1) {
        paths.enter()
            .append("path")
            .attr("class", "dataPath_" + id)
            .attr("stroke", this.pointAttributes[id].stroke)
            .style("fill", none);

        paths.attr("d", function () { return line(currentData) });
        paths.exit().remove();
    }else{
        paths.remove();
    }
}

BoxAndWhiskersMagnitude.prototype.updateDataFeatures = function(){
    this.xMin = this.xMax = this.yMin = this.yMax = undefined;
    Object.keys(this.data).forEach(function(id){
        if(this.toDisplay[id] != true){ return }
        xRange = d3.extent(this.data[id], function(d) { return d["xValue"] });
        this.xMin = (this.xMin == undefined) ? xRange[0] : d3.min([this.xMin, xRange[0]]);
        this.xMax = (this.xMax == undefined) ? xRange[1] : d3.max([this.xMax, xRange[1]]);

        yMin = d3.min(this.data[id], function(d) { return d["minValue"] });
        yMax = d3.max(this.data[id], function(d) { return d["maxValue"] });
        //yMin = d3.min(this.data[id], function(d) { return d["lowDevValue"] });
        //yMax = d3.max(this.data[id], function(d) { return d["highDevValue"] });

        yRange = d3.extent(this.data[id], function(d) { return d.value});
        this.yMin = (this.yMin == undefined) ? yMin : d3.min([this.yMin, yMin]);
        this.yMax = (this.yMax == undefined) ? yMax : d3.max([this.yMax, yMax]);
        }.bind(this));

    this.updateDomains([this.xMin, this.xMax], [this.yMin, this.yMax]);
}

BoxAndWhiskersMagnitude.prototype.updateDomains = function(xDomain, yDomain){
    
    var xValLeft, xValRight;
    range = xDomain[1] - xDomain[0];
    if(this.pad && range > 0){
        pad = 0.1 * range;
        xValLeft = xDomain[0] - pad;
        xValRight = xDomain[1] + pad;
    }
    else if (this.pad){
        pad = xDomain[1] > 10 ? 10 : xValMax;
        xValLeft = xDomain[0] - pad;
        xValRight = xDomain[0] + pad;
    }else{
        xValLeft = xDomain[0];
        xValRight = xDomain[1];
    }
    

	this.dXScale.range([0, this.width]).domain([xValLeft, xValRight]);
	this.dYScale.range([this.height, 0]).domain([0, this.yMax]);

	this.dYAxis.scale(this.dYScale);
	this.svg.select(".y.axis").call(this.dYAxis);

	this.dXAxis.scale(this.dXScale);
    this.svg.select(".x.axis").call(this.dXAxis);
}

BoxAndWhiskersMagnitude.prototype.getNextGroups = function(id){
	currentData = [];
	var groupNextLeft = {}, groupNextRight = {};
    var dataNextLeft = {}, dataNextRight = {};
    
   	this.data[id].forEach(function(dataPoint){
        if(this.toDisplay[id] != true){ return }
       
        if (dataPoint["xValue"] < this.dXScale.domain()[0]){

            if(groupNextLeft["xValue"] != undefined && groupNextLeft["xValue"] > dataPoint["xValue"]){ return }
            else{ groupNextLeft["xValue"] = dataPoint["xValue"]; groupNextLeft["medianValue"] = dataPoint["medianValue"] }

        }else if (dataPoint["xValue"] > this.dXScale.domain()[1]){

            if(groupNextRight["xValue"] != undefined && groupNextRight["xValue"] < dataPoint["xValue"]){ return }
            else{ groupNextRight["xValue"] = dataPoint["xValue"]; groupNextRight["medianValue"] = dataPoint["medianValue"] }

        }else{
            currentData.push(dataPoint);
        }
	}.bind(this));
    
    currentData.sort(function(a, b){ return a["xValue"] - b["xValue"] });
    var leftMost = currentData.length > 0 ? currentData[0] : groupNextRight;
    var rightMost = currentData.length > 0 ? currentData[currentData.length - 1] : groupNextLeft;

    //console.log(this.dXScale.domain());
    if(groupNextLeft["xValue"] != undefined){
        var slope = (groupNextLeft["medianValue"] - leftMost["medianValue"]) / (groupNextLeft["xValue"] - leftMost["xValue"]);
        var dx = this.dXScale.domain()[0] - leftMost["xValue"];
        dataNextLeft["xValue"] = this.dXScale.domain()[0];
        dataNextLeft["medianValue"] = leftMost["medianValue"] + dx * slope;
    }

    if(groupNextRight["xValue"] != undefined){
        var slope = (groupNextRight["medianValue"] - rightMost["medianValue"]) / (groupNextRight["xValue"] - rightMost["xValue"]);
        var dx = this.dXScale.domain()[1] - rightMost["xValue"];
        //console.log("Group:", groupNextRight, ", RightMost:", rightMost, ", Slope:", slope, ", dx:", dx, ", dataNextLeft:", groupNextLeft, ", leftMost:", leftMost);
        dataNextRight["xValue"] = this.dXScale.domain()[1];
        dataNextRight["medianValue"] = rightMost["medianValue"] + dx * slope;
    }
	
	return [dataNextLeft, dataNextRight];
}


//--------------------------------
// Time Plot Magnitude
// -------------------------------

TimePlotMagnitude.prototype = new BaseMagnitude();
TimePlotMagnitude.prototype.constructor = TimePlotMagnitude;

function TimePlotMagnitude(margins, width, height, selectedCallBack, parentDomId, displayPoints){
    BaseMagnitude.apply(this);
    this.init(margins, width, height, selectedCallBack, parentDomId);

    this.displayPoints = displayPoints == undefined ? true : displayPoints;

    this.line = d3.svg.line()
        .x(function(d) { return this.jMargin.left + this.dXScale(d.time) }.bind(this) )
        .y(function(d) { return this.jMargin.top + this.dYScale(d.value) }.bind(this) );

}

TimePlotMagnitude.prototype.display = function(id){
    if(this.toDisplay[id] != true) { 
        this.svg.selectAll(".circle_" + id).remove();
        this.svg.selectAll(".path_" + id).remove();
        return;
    }

    var paths = this.svg.selectAll(".path_" + id).data([this.data[id]]);

    paths.enter().append("path")

    paths
        .attr("d", function(d){ return this.line(d) }.bind(this))
        .attr("clip-path", "url(#" + this.id + "_clip)" )
        .attr("class", "path_" + id)
        .attr({"stroke-width":"0.5", "stroke":this.pointAttributes[id].stroke });

    paths.exit().remove();

    if(this.displayPoints == true){
        var circles = this.svg.selectAll(".circle_" + id).data(this.data[id]);
        circles.enter().append("circle").attr("class", "circle_" + id).attr(this.pointAttributes[id])
            .attr("clip-path", "url(#" + this.id + "_clip)" );
        
        var counter = 0;
        circles.each(function(d) { if(d.time > this.dXScale.domain()[0] && d.time < this.dXScale.domain()[1]) counter++; }.bind(this));
        var scaleFactor = this.getScaleFactor(10, counter);
        var position = 0;

        circles 
            .attr("cx", function(d) { return this.jMargin.left + this.dXScale(d.time) }.bind(this))
            .attr("cy", function(d) { return this.jMargin.top + this.dYScale(d.value) }.bind(this))
            .attr("r", 1.5)
            .attr(this.pointAttributes[id])
            .attr("visibility", function(d, i){
                var t;
                if((i % scaleFactor == 0) && this.dXScale(d.time) >= position) {
                    t = "visible";
                    position = this.dXScale(d.time) + 5;
                } else {
                    t = "hidden";
                }
                return t;
                }.bind(this))
            .on(this.mouseEvents[id]);

       circles.exit().remove();
    }
}

TimePlotMagnitude.prototype.addDataSet = function(id, data, pointAttributes, mouseEvents){
    data.sort(function(a, b){ return a.time - b.time });
    this.data[id] = data;
    this.pointAttributes[id] = pointAttributes;
    this.mouseEvents[id] = mouseEvents;
    this.toDisplay[id] = true;
    this.updateDataFeatures();
}

TimePlotMagnitude.prototype.getScaleFactor = function(pointsPerPixel, size) {
    return Math.ceil((size / this.width) * pointsPerPixel);
}
