
// Layout components
var marginFocus, focusWidth, contextWidth, focusHeight, contextHeight;
// Scaling 
var xScaleFocus, yScaleFocus, xScaleContext, yScaleContext;
// Axes to be used for the graph
var xAxisFocus, yAxisFocus, xAxisContext, yAxisContext;
// Areas where things will be graphed
var svg, context, focus, brush;
// Our data, this will need to change
var metricList, metricData, metricTypes, metricLoaded;
// Inportant details about the data
var xValMin, xValMax, yValMin, yValMax;
// Information about the collective data groups
var metricGroupData, dataGroupNumber, dataGroupSize;
// The name of the independent variable / dependent variables (Array)
var xKey, yKey, yKeyColors, colors;
// Drawing details
var contextDataPointRadius, focusDataPointRadius;
// Used to display data information
var infoMessage, numberFormat;
var srun_id = 1;
var exp_id = 1;
var crossSelect;

// Initializes graph componenets and variables
function initVisualComponents(){
	// Configure margins
	marginFocus = {top: 40, right: 40, bottom: 20, left: 40};
	marginContext = { top: 20, right: marginFocus.right, bottom: 20, left: 40 };
	focusWidth = 960 - marginFocus.left - marginFocus.right;
	focusHeight = 500 - marginFocus.top - marginFocus.bottom;
	contextWidth = focusWidth;
	contextHeight = 150 - marginContext.top - marginContext.bottom;

	// Configure time scales
	xScaleFocus = d3.scale.linear().range([0, focusWidth]).domain([0,0]);
	yScaleFocus = d3.scale.linear().range([focusHeight, 0]).domain([0,0]);
	xScaleContext = d3.scale.linear().range([0, contextWidth]).domain([0, 0]);
	yScaleContext = d3.scale.linear().range([contextHeight, 0]).domain([0, 0]);

	// Configure axis
	xAxisFocus = d3.svg.axis().scale(xScaleFocus).orient("bottom");
	yAxisFocus = d3.svg.axis().scale(yScaleFocus).orient("left");
	xAxisContext = d3.svg.axis().scale(xScaleContext).orient("bottom");
	yAxisContext = d3.svg.axis().scale(yScaleContext).orient("left");

    // Add focus plot
	focus = d3.select("#chart1")
        .append("svg")
        .attr("width", focusWidth + marginFocus.left + marginFocus.right)
        .attr("height", focusHeight + marginFocus.top + marginFocus.bottom );

    // Add context plot
    context = d3.select("#chart2")
        .append("svg")
        .attr("width", contextWidth + marginContext.left + marginContext.right)
        .attr("height", contextHeight + marginContext.top + marginContext.bottom );

	// Text label for x axis
	context.append("text")
        .attr("class", "x label")
		.attr("x", contextWidth / 2)
		.attr("y", contextHeight)
		.attr("dy", "3em")
		.style("text-anchor", "middle")
		.text("Original x-axis");

	// Add text label for y axis
	focus.append("text")
		.attr("class", "y label")
		.attr("transform", "rotate(-90)")
		.attr("y", 0)
		.attr("x", 0 - ((focusHeight + contextHeight) / 2))
		.attr("dy", "1em")
		.style("text-anchor", "middle")
		.text("Original y-axis");

	// Add x axis
	focus.append("g")
		.attr("class", "x axis focus")
		.attr("transform", "translate(" + marginFocus.left + "," + (focusHeight + marginFocus.top) + ")")
		.call(xAxisFocus);

	context.append("g")
        .attr("class", "x axis context")
        .attr("transform", "translate(" + marginContext.left + "," + (contextHeight + marginContext.top) + ")")
        .call(xAxisContext);

	// Add y axis
    focus.append("g")
		.attr("class", "y axis focus")
		.attr("transform", "translate(" + marginFocus.left + "," + marginFocus.top + ")")
		.call(yAxisFocus);

    context.append("g")
		.attr("class", "y axis context")
		.attr("transform", "translate(" + marginContext.left + "," + marginContext.top + ")")
		.call(yAxisContext);

	// Add the information bar
	infoMessage = d3.select("body")
        .append("div")
		.attr("class", "tooltip")
        .style("opacity", 0);

    brush = d3.svg.brush() 
        .x(xScaleContext)
        .on("brush", brushed);

    context.append("g")
        .attr("class", "brush")
        .attr("transform", "translate(" + marginContext.left + "," + marginContext.top + ")")
        .call(brush)
        .selectAll("rect")
            .attr("y", -6)
            .attr("height", contextHeight + 7);

	// Miscilanious variables
	yKey = [];
	xKey = null;
	yKeyColors = [];
	metricData = [];
    metricLoaded = [];
    metricGroupData = [];
    dataGroupNumber = 10;
    colors = ["blue", "green", "orange"];
	contextDataPointRadius = 1.5;
	focusDataPointRadius = 3;
	numberFormat = d3.format(".2f");
}

// Initializes the key for the graph
function initGraphKey(){
    d3.select("#key")
        .append("input")
        .attr("type", "number")
        .attr("max", "100")
        .attr("min", "1")
        .on("change", dataGroupNumberChange)
        .attr("value", dataGroupNumber);

	// Add a selector for the x-axis (independent variable)
	var select = d3.select("#key")
		.append("select")
		.attr("id", "xAxisSelect")
		.on("change", xAxisSelectChange);

	metricTypeList = d3.set();
	Object.keys(metricList).forEach(function (key) {
	    d = metricList[key];
	    if (!metricTypeList.has(d.metric_typeA)) {
	        metricTypeList.add(d.metric_typeA);
	    }
	    if (!metricTypeList.has(d.metric_typeB)) {
	        metricTypeList.add(d.metric_typeB);
	    }
	});

	// Populate the selector
	select.selectAll("option")
		.data(metricTypeList.values())
		.enter()
			.append("option")
			.text(function(d) { return d; });

    // Initialize independent variable
	xAxisSelectChange.call(document.getElementById("xAxisSelect"));

	// Add a new key line for each metric, binding its metric_id to id
	var key_items = d3.select("#key") 
		.selectAll("div") 
		.data(Object.keys(metricList))
		.enter()
			.append("div") 
			.attr("class","key_line") 
			.attr("metric_id", function(d){ return metricList[d].metric_id })
			.attr("id",function(d){return "key_" + metricList[d].metric_id });

	// Add a new key square for each metric
	key_items.append("div") 
		.attr("id", function(d){ return "key_square_" + metricList[d].metric_id}) 
		.attr("class", "key_square");

	// Add a label for each metric
	key_items.append("div")
		.attr("class","key_label")
        .attr("id", function(d){ return "key_label_" + metricList[d].metric_id }) 
		.text(function(d){return metricList[d].metric_name});

	// Listen for clicks to turn on/off the time series
	d3.selectAll(".key_line")
		.on("click", keyElementSelectChange);

	var crossHairEnable = d3.selectAll("#options")
        .append("div")
        .attr("id", "crossHairEnable");

	crossHairEnable.append("input")
        .attr("type", "checkbox")
        .style("float", "left")
        .on("change", enableCrossHairs);

	crossHairEnable.append("p")
        .text("Cross Hairs");

}

// Reads metric information (currently from file)
function readInitialData(){
    //d3.json("../data/simrun=" + srun_id + ".txt", function (error, json) {
    d3.json("/safe/exp-descs/" + exp_id + "/metrics", function(error, json) {
        if (error) return console.warn(error);

        metricList = [];
        metricTypes = [];
        json.forEach(function (metric) {
            metricList[metric.metric_id] = metric;
            if (!(metric.typeA in metricTypes)) { metricTypes.push(metric.typeA); }
            if (!(metric.typeB in metricTypes)) { metricTypes.push(metric.typeB); }
        });

        initGraphKey();
        updateDataFeatures();
        updateFromIndependentVariable();
        displayData();
    });
}

// Reads metric information (currently from file)
function readMetric(metric_id){
    //d3.json("../data/simrun=" + srun_id + "&metric=" + metric_id + ".txt", function (error, json) {
    d3.json("/safe/sim-run/" + srun_id + "/metric/" + id.slice(1,id.length) + "/results/", function (error, json) {
        if (error) {
            removeDependentVariable(metric_id);
            return console.warn(error);
        }
        metricData[metric_id] = json;
        metricLoaded[metric_id] = true;

        updateDataFeatures();
        calculateDataGroups();
        adjustScaling();
        displayData();
    });
}

// Displays the currently held data to the graph
function displayData(){
	// Can't draw until data is specified
	if(xKey == null || yKey.length == 0){	
		return;
	}

	yKey.forEach(function (metric_id) {
        updateContextDataPoints(metric_id);
	});
}

// Updates the positions of the center points described by the data groupings
function updateContextDataPoints(metric_id){
    currentData = metricGroupData[metric_id];
    var currentYKey = getYKey(metric_id);
    
    //=====================================
    contextGroups = context.selectAll(".contextDataGroup_" + metric_id).data(currentData);
    contextGroups.enter().append("g").attr("class", "contextDataGroup_" + metric_id)
	contextGroups.exit().remove();

    //=====================================
    contextRanges = contextGroups.selectAll(".contextDataRange_" + metric_id)
        .data(["yValueMax", "yValueMin", "yValueMid", "yValueDevMin", "yValueDevMax"]);

    contextRanges.transition()
        .attr("cx", function (d, e, f) { return marginContext.left + xScaleContext(currentData[f]["xValue"]) })
        .attr("cy", function (d, e, f) { return marginContext.top + yScaleContext(currentData[f][d]) });

    contextRanges.enter()
        .append("circle")
        .attr("class", "contextDataRange_" + metric_id)
        .attr("stroke", yKeyColors[metric_id])
        .attr("fill", "none")
        .attr("r", contextDataPointRadius)
        .attr("cx", function (d, e, f) { return marginContext.left + xScaleContext(currentData[f]["xValue"]) })
        .attr("cy", function (d, e, f) { return marginContext.top + yScaleContext(currentData[f][d]) });

	contextRanges.exit()
        .remove();

    //=====================================
    contextLines = contextGroups.selectAll(".contextDataLine_" + metric_id).data([""])

    contextLines.transition()
        .attr("x1", function (d, e, f) { return marginContext.left + xScaleContext(currentData[f]["xValue"]); })
        .attr("x2", function (d, e, f) { return marginContext.left + xScaleContext(currentData[f]["xValue"]); })
        .attr("y1", function (d, e, f) { return marginContext.top + yScaleContext(currentData[f]["yValueDevMax"]); })
        .attr("y2", function (d, e, f) { return marginContext.top + yScaleContext(currentData[f]["yValueDevMin"]); });

    contextLines.enter()
        .append("line")
        .attr("class", "contextDataLine_" + metric_id)
        .attr("stroke", yKeyColors[metric_id])
        .attr("stroke-width", "0.5")
        .attr("stroke-dasharray", "5,5")
        .attr("x1", function (d, e, f) { return marginContext.left + xScaleContext(currentData[f]["xValue"]); })
        .attr("x2", function (d, e, f) { return marginContext.left + xScaleContext(currentData[f]["xValue"]); })
        .attr("y1", function (d, e, f) { return marginContext.top + yScaleContext(currentData[f]["yValueDevMax"]); })
        .attr("y2", function (d, e, f) { return marginContext.top + yScaleContext(currentData[f]["yValueDevMin"]); });

    contextLines.exit()
        .remove();

    //=====================================
    line = d3.svg.line()
        .x(function (d) { return marginContext.left + xScaleContext(d["xValue"]) })
        .y(function (d) { return marginContext.top + yScaleContext(d["yValueMid"]) });

    paths = context.selectAll(".contextDataPath_" + metric_id).data([""]);
    
    paths.transition()
        .attr("d", function(){ return line(metricGroupData[metric_id]) });

    paths.enter()
        .append("path")
        .attr("d", function(){ return line(metricGroupData[metric_id]) })
        .attr("class", "contextDataPath_" + metric_id)
        .attr("stroke", yKeyColors[metric_id])
        .attr("stroke-width", 1);

    paths.exit()
        .remove();
}

function updateFocusDataPoints(metric_id){
    var currentData = [];
    var dataNextLeft = [], dataNextRight = [];

    metricGroupData[metric_id].forEach(function (d, i) {
        if (xScaleFocus.domain()[0] > d["xValue"]) {
            var e = metricGroupData[metric_id][i + 1];
            if (e == undefined) { return }
            slope = (e["yValueMid"] - d["yValueMid"]) / (e["xValue"] - d["xValue"]);
            var dx = xScaleFocus.domain()[0] - d["xValue"];
            dataNextLeft["xValue"] = xScaleFocus.domain()[0];
            dataNextLeft["yValueMid"] = d["yValueMid"] + dx * slope;
        } else if (xScaleFocus.domain()[1] < d["xValue"]) {
            if (dataNextRight.xValue == undefined) {
                var e = metricGroupData[metric_id][i - 1];
                if (e == undefined) { return }
                slope = (d["yValueMid"] - e["yValueMid"]) / (d["xValue"] - e["xValue"]);
                var dx = xScaleFocus.domain()[1] - d["xValue"];
                dataNextRight["xValue"] = xScaleFocus.domain()[1];
                dataNextRight["yValueMid"] = d["yValueMid"] + dx * slope;
            }
        } else {
            currentData.push(d);
        }
    })

    //=====================================
    focusGroups = focus.selectAll(".focusDataGroup_" + metric_id).data(currentData);
    focusGroups.enter().append("g").attr("class", "focusDataGroup_" + metric_id)
	focusGroups.exit().remove();

    //=====================================
    focusPoints = focusGroups.selectAll(".focusDataRange_" + metric_id)
        .data(["yValueMin", "yValueMid", "yValueMax", "yValueDevMax", "yValueDevMin"]);
    
    focusPoints
        .attr("cx", function (d, e, f) { return marginFocus.left + xScaleFocus(currentData[f]["xValue"]) })
        .attr("cy", function (d, e, f) { return marginFocus.top + yScaleFocus(currentData[f][d]) })
        .attr("info", function(d, e, f){ return numberFormat(currentData[f]["xValue"]) +
                                  "<br/>" + numberFormat(currentData[f][d]); });

    focusPoints.enter()
        .append("circle")
        .attr("class", "focusDataRange_" + metric_id)
        .attr("stroke", yKeyColors[metric_id])
        .attr("fill", "white")
        .attr("fill-opacity", 0)
        .attr("r", focusDataPointRadius)
        .attr("cx", function (d, e, f) { return marginFocus.left + xScaleFocus(currentData[f]["xValue"]) })
        .attr("cy", function (d, e, f) { return marginFocus.top + yScaleFocus(currentData[f][d]) })
        .attr("info", function(d, e, f){ return numberFormat(currentData[f]["xValue"]) +
                                  "<br/>" + numberFormat(currentData[f][d]); })
        .on("mouseover", mouseOverDataPoint)
        .on("mouseout", mouseOutDataPoint);

    focusPoints.exit()
        .remove();

    //=====================================
    focusLines = focusGroups.selectAll(".focusDataLine_" + metric_id).data([""])

    focusLines
        .attr("x1", function (d, e, f) { return marginFocus.left + xScaleFocus(currentData[f]["xValue"]); })
        .attr("x2", function (d, e, f) { return marginFocus.left + xScaleFocus(currentData[f]["xValue"]); })
        .attr("y1", function (d, e, f) { return marginFocus.top + yScaleFocus(currentData[f]["yValueDevMax"]); })
        .attr("y2", function (d, e, f) { return marginFocus.top + yScaleFocus(currentData[f]["yValueDevMin"]); });

    focusLines.enter()
        .append("line")
        .attr("class", "focusDataLine_" + metric_id)
        .attr("stroke", yKeyColors[metric_id])
        .attr("stroke-dasharray", "5,5")
        .attr("x1", function (d, e, f) { return marginFocus.left + xScaleFocus(currentData[f]["xValue"]); })
        .attr("x2", function (d, e, f) { return marginFocus.left + xScaleFocus(currentData[f]["xValue"]); })
        .attr("y1", function (d, e, f) { return marginFocus.top + yScaleFocus(currentData[f]["yValueDevMax"]); })
        .attr("y2", function (d, e, f) { return marginFocus.top + yScaleFocus(currentData[f]["yValueDevMin"]); });

    focusLines.exit()
        .remove();

    //=====================================
    line = d3.svg.line()
        .x(function (d) { return marginFocus.left + xScaleFocus(d["xValue"]) })
        .y(function (d) { return marginFocus.top + yScaleFocus(d["yValueMid"]) });

    if (dataNextLeft.xValue != undefined) { currentData.unshift(dataNextLeft); }
    if (dataNextRight.xValue != undefined) { currentData.push(dataNextRight); }

    paths = focus.selectAll(".focusDataPath_" + metric_id).data([""]);

    if (currentData.length > 0) {
        paths
            .attr("d", function () { return line(currentData) });

        paths.enter()
            .append("path")
            .attr("d", function () { return line(currentData) })
            .attr("class", "focusDataPath_" + metric_id)
            .attr("stroke", yKeyColors[metric_id])
            .attr("stroke-width", 1);

        paths.exit()
            .remove();
    }else{
        paths.remove();
    }
}

// Called when a new dependent variable is selected. Updates required data
function xAxisSelectChange(){
    setIndependentVariable(this.options[this.selectedIndex].value);
    updateFromIndependentVariable();
    metricGroupData = [];
    updateDataFeatures();
    adjustScaling();
    displayData();
}

// Called when a key element is selected. Updates required data
function keyElementSelectChange(){
	metric_id = this.getAttribute("metric_id");
	index = yKey.indexOf(metric_id);
	if(index < 0){
	    if (addDependentVariable(metric_id)) {
	        var d = document.getElementById("key_square_" + metric_id)
                .style.background = yKeyColors[metric_id];
	    }
	}else{
		removeDependentVariable(metric_id);
		document.getElementById("key_square_" + metric_id)
            .style.background = "white";
	}
}

// Called when the mouse is positioned over a data element
function mouseOverDataPoint(){
    infoMessage.transition()        
                .duration(200)      
                .style("opacity", .9);      
    infoMessage.html(this.getAttribute("info"))  
        .style("left", (d3.event.pageX) + "px")     
        .style("top", (d3.event.pageY - 28) + "px");
}

// Called when the mouse is no longer over a data element
function mouseOutDataPoint(){
	infoMessage.transition()
        .duration(500)
        .style("opacity", 0);
}

// Sets the current independent variable
function setIndependentVariable(name){
	xKey = name;
	d3.select(".x.label")
        .text(xKey);
}

// Adds the given key to the list to be graphed
function addDependentVariable(metric_id){
    if (!(metricList[metric_id].metric_typeA == xKey) && !(metricList[metric_id].metric_typeB == xKey))
        return false;
	yKey.push(metric_id);
    if(metricLoaded[metric_id] == undefined){
        yKeyColors[metric_id] = colors.shift();
        readMetric(metric_id);
    }else{
        updateDataFeatures();
	    adjustScaling();
	    displayData();
    }
    return true;
}

// Determines which metrics should remain visible and/or selectable
function updateFromIndependentVariable(){
    yKey.forEach(function (metric_id) {
        if (metricList[metric_id].metric_typeA != xKey && metricList[metric_id].metric_typeB != xKey) {
            removeDependentVariable(metric_id);
            document.getElementById("key_square_" + metric_id)
                .style.background = "white";
        }});
        Object.keys(metricList).forEach(function (metricKey) {
            metric = metricList[metricKey];
            if (metric.metric_typeA != xKey && metric.metric_typeB != xKey) {
                d3.select("#key_label_" + metric.metric_id)
                    .style("color", "rgb(200, 200, 200)");
            } else {
                d3.select("#key_label_" + metric.metric_id)
                    .style("color", "black");
            }
        });
}

// Removes the specified key fro the list of dependent variables (no longer displayed)
function removeDependentVariable(key){
	index = yKey.indexOf(key);
	yKey.splice(index, 1);
	context.selectAll(".contextDataGroup_" + key).remove();
	context.selectAll(".contextDataPath_" + key).remove();
	focus.selectAll(".focusDataGroup_" + key).remove();
	focus.selectAll(".focusDataPath_" + key).remove();
	updateDataFeatures();
	adjustScaling();
	displayData();
}

// Adjusts the scales base on current data values, dependent, and independent variables
function adjustScaling(){
	xScaleContext.range([0, contextWidth]).domain([0,xValMax]);
	yScaleContext.range([contextHeight, 0]).domain([0,yValMax]);

	yAxisContext.scale(yScaleContext);
	d3.select(".y.axis.context").transition().call(yAxisContext);

	xAxisContext.scale(xScaleContext);
    d3.select(".x.axis.context").transition().call(xAxisContext);

    brushed();
}

// Updates important information (such as min/max values)
function updateDataFeatures(){
    xValMin = -1;
    xValMax = -1;
    yValMin = -1;
    yValMax = -1;
    yKey.forEach(function (metric_id) {
        var currentYKey = getYKey(metric_id);
        if (currentYKey != undefined) {
            xValRange = d3.extent(metricData[metric_id], function (d) { return d[xKey]; });
            xValMin = (xValMin < 0) ? xValRange[0] : d3.min([xValMin, xValRange[0]]);
            xValMax = (xValMax < 0) ? xValRange[1] : d3.max([xValMax, xValRange[1]]);

            yValRange = d3.extent(metricData[metric_id], function (d) { return d[currentYKey]; });
            yValMin = (yValMin) < 0 ? yValRange[0] : d3.min([yValMin, yValRange[0]]);
            yValMax = (yValMax) < 0 ? yValRange[1] : d3.max([yValMax, yValRange[1]]);
        }
    });
    if (yKey.length == 0)
        xValMin = xValMax = yValMin = yValMax = 0;
    calculateDataGroups();
}

function brushed() {
    var extent = brush.extent();
    xScaleFocus.domain(brush.empty() ? xScaleContext.domain() : [ extent[0], extent[1] ]);
    yScaleFocus.domain(yScaleContext.domain());

    focus.select(".x.axis.focus").call(xAxisFocus);
    focus.select(".y.axis.focus").call(yAxisFocus);

    yKey.forEach(function (metric_id) {
        updateFocusDataPoints(metric_id);
    });
}

function calculateDataGroups(){
    dataGroupSize = (xValMax + 1) / dataGroupNumber;
    yKey.forEach(function (metricKey) {
        metric = metricList[metricKey];
        if (metricGroupData[metric.metric_id] == undefined ) {
            metricGroupData[metric.metric_id] = calculateMetricGroupData(metric);
        }
    });
}

function calculateMetricGroupData(metric){
    var groupInfo = [];
    var groupData = [];
    for (var i = 0; i < dataGroupNumber; i++) { groupInfo.push([]); groupData.push([]);  }

    var data = metricData[metric.metric_id];
    var currentYKey = getYKey(metric.metric_id);

    data.forEach(function (element) {
        var group = Math.floor(element[xKey] / dataGroupSize);
        groupData[group].push(element);
    })

    groupData.forEach(function (element, index) {
        groupInfo[index]["xValue"] = d3.mean(element, function (d) { return d[xKey] });

        var mySort = element.slice().sort(function (a, b) { return a[currentYKey] - b[currentYKey]; });
        var sorted = [];
        mySort.forEach(function (element) { sorted.push(element[currentYKey]) });
        groupInfo[index]["yValueMin"] = sorted[0];
        groupInfo[index]["yValueMax"] = sorted[sorted.length - 1];
        groupInfo[index]["yValueMid"] = d3.quantile(sorted, 0.5);
        groupInfo[index]["yValueDevMax"] = d3.quantile(sorted, 0.75);
        groupInfo[index]["yValueDevMin"] = d3.quantile(sorted, 0.25);
    });
    return groupInfo;
}

function getYKey(metric_id){
    if(metricList[metric_id].metric_typeA == xKey){ return metricList[metric_id].metric_typeB; }
    else if(metricList[metric_id].metric_typeB == xKey){ return metricList[metric_id].metric_typeA; }
    else{ return undefined; }
}

function dataGroupNumberChange(){
    if (Number(this.value) > Number(this.max)){
        this.value = this.max;
    }else if (Number(this.value) < Number(this.min))
        this.value = this.min;
    dataGroupNumber = this.value;
    metricGroupData = [];
    calculateDataGroups();
    adjustScaling();
    displayData();
}


function updateCrossHairs(){
    var x = d3.event.offsetX;
    var y = d3.event.offsetY;

    var xLeft = xScaleFocus.range()[0] + marginFocus.left; 
    var xRight = xScaleFocus.range()[1] + marginFocus.left;
    var yTop = yScaleFocus.range()[0] + marginFocus.top;
    var yBottom = yScaleFocus.range()[1] + marginFocus.top;

    if (x < xLeft) { x = xLeft }
    if (x > xRight) { x = xRight }
    if (y < yBottom) { y = yBottom }
    if (y > yTop) { y = yTop }

    horizontal = focus.selectAll(".crossHairHorizontal").data([""]);
    vertical = focus.selectAll(".crossHairVertical").data([""]);

    horizontal.attr({"x1": xLeft, "x2": xRight,"y1": y, "y2": y});
    vertical.attr({"y1": yBottom, "y2": yTop, "x1": x, "x2": x});

    horizontal.enter().append("line").style({ "stroke": "black", "pointer-events": "none" })
        .attr({"class": "crossHairHorizontal", "x1": xLeft, "x2": xRight, "y1": y, "y2": y});
    vertical.enter().append("line").attr("class", "crossHairVertical").style("stroke", "black").style("pointer-events", "none")
        .attr("y1", yBottom).attr("y2", yTop).attr("x1", x).attr("x2", x);

    horizontal.exit().remove();
    vertical.exit().remove();
}

function enableCrossHairs(){
    if(this.checked){
        focus.on("mousemove", updateCrossHairs);
    }else{
        focus.selectAll(".crossHairHorizontal").remove();
        focus.selectAll(".crossHairVertical").remove();
        focus.on("mousemove", undefined);
    }
}

initVisualComponents();
readInitialData();
