//----------------------------------------------
// Full Selection Object Prototypes
// ---------------------------------------------

// Initializes the selection for the graph
// callBack of the form callBack(run_id, metric_id)
function SelectionFull(){
}

selectionFullNumber = 0;
SelectionFull.prototype.init = function(exp_id, selectionCallBack, applyFilterCallBack, parentDomId){
    this.exp_id = exp_id;
    this.parentDomId = parentDomId;
    this.dParent = d3.select("#" + parentDomId);
    console.log("Parent:", this.dParent);
    this.eParent = document.getElementById(parentDomId);
    this.selectionCallBack = selectionCallBack;
    this.applyFilterCallBack = applyFilterCallBack;
    this.id = parentDomId + "_selection_full_" + selectionFullNumber;
    selectionFullNumber += 1;
    
    this.dSelf = this.dParent.append("div").attr("id", this.id);
    this.eSelf = document.getElementById(this.id);

    this.selectFactor = new SelectionFactor(this.exp_id, this.factorSelectChange.bind(this), this.id);
    this.selectLevelMin = new SelectionLevel(this.exp_id, undefined, this.id);
    this.selectLevelMax = new SelectionLevel(this.exp_id, undefined, this.id);
    this.selectDesignPoint = new SelectionDesignPoint(this.exp_id, this.designPointSelectChange.bind(this), this.id);
    this.selectRun = new SelectionRun(this.exp_id, this.runSelectChange.bind(this), this.id);
    this.selectMetric = new SelectionMetric(this.exp_id, this.selectElementChange.bind(this), this.id);

    this.selectFactor.readInitialData(function(){
        this.selectLevelMin.readInitialData(function(){
            this.selectLevelMax.readInitialData(function(){
                this.selectMetric.readInitialData(this.initComponents.bind(this))
            }.bind(this));
        }.bind(this));
    }.bind(this));
}
    
SelectionFull.prototype.initComponents = function(){
    // Initialize the factor list
    this.selectFactor.initComponents();
    this.selectLevelMin.initComponents();
    this.selectLevelMax.initComponents();
     
    // Initialize the apply selection button
    this.dSelf
        .append("button")
        .attr("class", "apply_selection")
        .text("Apply Selection Filter")
        .on("click", this.applyDesignFilter.bind(this));

    this.selectDesignPoint.initComponents();
    this.selectRun.initComponents();
    this.selectMetric.initComponents();
      
    this.factorSelectChange();
    this.updateDesignPoints();
}

SelectionFull.prototype.factorSelectChange = function(){

    var factor_id = this.selectFactor.getSelectedFactor().id;

    this.selectLevelMin.updateLevels(factor_id);
    this.selectLevelMax.updateLevels(factor_id);

    this.selectLevelMax.eSelect.selectedIndex = this.selectLevelMax.eSelect.options.length - 1;
}

SelectionFull.prototype.applyDesignFilter = function(){

    var factor = this.selectFactor.getSelectedFactor();
    var min = this.selectLevelMin.getMinLevel(factor.id);
    var max = this.selectLevelMax.getMaxLevel(factor.id);

    if(min > max){
        alert("The factor's beginning level cannot be greater than the factor's ending level");
		return;
    }

    var parameter = {"name":factor.name, "factor_id":factor.id, "min_value":this.getMinLevel(), "max_value":this.getMaxLevel() };
    if(this.getMinLevel() > min || this.getMaxLevel() < max ){
		this.selectDesignPoint.addSelectionParameter(factor.id, parameter);
	}else{
		this.removeDesignFilter(factor.id);
	}

    this.updateDesignPoints();
    this.applyFilterCallBack(parameter);
}

SelectionFull.prototype.removeDesignFilter = function(factor_id){
    this.selectDesignPoint.removeSelectionParameter(factor_id);

    this.updateDesignPoints();
}

SelectionFull.prototype.updateDesignPoints = function(){
	this.selectDesignPoint.updateDesignPoints(this.designPointSelectChange.bind(this));
}

SelectionFull.prototype.designPointSelectChange = function(){
    this.selectRun.updateRuns(this.selectDesignPoint.getSelectedDesignPoint(), function(){
        this.selectMetric.setDistinctionId(this.getSelectedRun());
        this.selectMetric.updateSelectedMetrics();  
    }.bind(this));
}

SelectionFull.prototype.runSelectChange = function(){
    this.selectMetric.setDistinctionId(this.getSelectedRun());
    this.selectMetric.updateSelectedMetrics();
}

SelectionFull.prototype.addDisplayed = function(run_id, metric_id, color){
    this.selectMetric.setMetricColor(run_id, metric_id, color);
    this.selectMetric.updateSelectedMetrics();
}

SelectionFull.prototype.getDesignParameters = function(){
    return this.selectDesignPoint.getSelectionParameters();
}

SelectionFull.prototype.removeDisplayed = function(run_id, metric_id){
    this.selectMetric.setMetricColor(run_id, metric_id, "white");
    this.selectMetric.updateSelectedMetrics();
}

SelectionFull.prototype.selectElementChange = function(){
    var metric_id = d3.event.toElement.getAttribute("metric_id");
    var metric = this.selectMetric.getMetric(metric_id);
    if(this.selectionCallBack){ this.selectionCallBack(this.getSelectedRun(), metric) }
}

SelectionFull.prototype.getSelectedFactor = function(){
    return this.selectFactor.getSelectedFactor();
}

SelectionFull.prototype.getMinLevel = function(){
    return this.selectLevelMin.getSelectedLevel();
}

SelectionFull.prototype.getMaxLevel = function(){
    return this.selectLevelMax.getSelectedLevel();
}

SelectionFull.prototype.getSelectedDesignPoint = function(){
    return this.selectDesignPoint.getSelectedDesignPoint();
}

SelectionFull.prototype.getSelectedRun = function(){
    return this.selectRun.getSelectedRun();
}


//----------------------------------------------
// Factor Selection Object Prototypes
// ---------------------------------------------?denoise_window=20&max_results=150

selectionFactorNumber = 0;
SelectionFactor = function(exp_id, changeCallBack, parentDomId){
    this.exp_id = exp_id;
    this.callBack = changeCallBack;
    this.dParent = d3.select("#" + parentDomId);
    this.eParent = document.getElementById(parentDomId);
    this.factorList = {};
	this.id = "selectionFactor_" + selectionFactorNumber;
	selectionFactorNumber += 1;
}

SelectionFactor.prototype.readInitialData = function(callBack){
    d3.json("/safe/exp/" + exp_id + "/factors/", function(error, data){
        if(error){ console.warn(error) }

        data.forEach(function(factor){ this.factorList[factor.id] = factor }.bind(this));
        if(callBack != undefined){ callBack() }
    }.bind(this));
}

SelectionFactor.prototype.initComponents = function(attributes, style){
    this.dSelect = this.dParent.selectAll("#" + this.id).data([""]).enter()
        .append("select")
        .attr("class", "factor_select")
		.attr("id", this.id)
		.style(style)
        .on("change", this.selectChange.bind(this));
    this.eSelect = document.getElementById(this.id);

    this.dSelect
        .selectAll("option")
        .data(Object.keys(this.factorList))
        .enter()
        .append("option")
        .attr("factor", function(d) { return d })
        .text(function(d) { return this.factorList[d].name }.bind(this));
}

SelectionFactor.prototype.selectChange = function(){
    if(this.callBack != undefined){ this.callBack(this.getSelectedFactor()) }
}

SelectionFactor.prototype.getSelectedFactor = function(){
    var factor_id = this.eSelect.options[this.eSelect.selectedIndex].getAttribute("factor");
    return this.factorList[factor_id];
}

//----------------------------------------------
// Level Selection Object Prototypes
// ---------------------------------------------

selectionLevelNumber = 0;
SelectionLevel = function(exp_id, changeCallBack, parentDomId){
    this.exp_id = exp_id;
    this.callBack = changeCallBack;
    this.dParent = d3.select("#" + parentDomId);
    this.eParent = document.getElementById(parentDomId);
    this.levelList = {};
    this.id = "selectionLevel_" + selectionLevelNumber;
	selectionLevelNumber += 1; 
}

SelectionLevel.prototype.readInitialData = function(callBack){
    d3.json("/safe/exp/" + exp_id + "/levels/", function(error, data){
        if(error){ console.warn(error) }

        Object.keys(data).forEach(function(factor_id){ 
            this.levelList[factor_id] = data[factor_id].levels.sort(); 
        }.bind(this));

        if(callBack != undefined){ callBack() }
    }.bind(this));
}

SelectionLevel.prototype.initComponents = function(attributes, styles){
    this.dSelect = this.dParent.selectAll("#" + this.id).data([""]).enter()
        .append("select")
        .attr(attributes)
        .attr("class", "level_select")
		.attr("id", this.id)
        .style(styles)
		.on("change", this.selectChange.bind(this));
    this.eSelect = document.getElementById(this.id);
}

SelectionLevel.prototype.updateLevels = function(factor_id){
    levels = this.dSelect.selectAll("option").data(this.levelList[factor_id])
    levels.enter().append("option");
    levels.attr("level", function(d) { return d }).text(function(d) { return d });
    levels.exit().remove();
    this.eSelect.selectedIndex = 0;
}

SelectionLevel.prototype.setLevels = function(factor_id, levels){
    this.levelList[factor_id] = levels;
}

SelectionLevel.prototype.selectChange = function(){
    if(this.callBack){ this.callBack(this.getSelectedLevel()) }
}

SelectionLevel.prototype.getSelectedLevel= function(){
    return this.eSelect.options[this.eSelect.selectedIndex].getAttribute("level");
}

SelectionLevel.prototype.getMinLevel = function(factor_id){
    return this.levelList[factor_id][0];
}

SelectionLevel.prototype.getMaxLevel = function(factor_id){
    return this.levelList[factor_id][this.levelList[factor_id].length - 1 ];
}

//----------------------------------------------
// Design Point Selection Object Prototypes
// ---------------------------------------------

selectionDesignPointNumber = 0;
SelectionDesignPoint = function(exp_id, changeCallBack, parentDomId){
    this.exp_id = exp_id;
    this.callBack = changeCallBack;
    this.dParent = d3.select("#" + parentDomId);
    this.eParent = document.getElementById(parentDomId);
    this.id = "selectionDesignPoint_" + selectionDesignPointNumber;
	selectionDesignPointNumber += 1;

    this.designPointList = {};
    this.designPointFactors = undefined;
    this.parameters = {};

}

SelectionDesignPoint.prototype.readInitialData = function(callBack){
    //console.log("Reading initial data...");
    this.designPointFactors = {};
    d3.json("/safe/exp/" + this.exp_id + "/designpoints/", function(error, designPoints){
        designPoints.forEach(function(designPoint){
            this.designPointFactors[designPoint.id] = designPoint.factors;
        }.bind(this));
        //console.log(this.designPointFactors);
        if(callBack != undefined){ callBack() }
    }.bind(this));
}

SelectionDesignPoint.prototype.readData = function(callBack){
    var update = function(){ 
        var parameters = this.getSelectionParameters();

        d3.json("/safe/exp/" + this.exp_id + "/dp-range-query/").post(JSON.stringify(parameters), function(error, designPoints){
            this.designPointList = designPoints;
            if(callBack != undefined){ callBack() };
        }.bind(this));
    }.bind(this)
    
    if(this.designPointFactors == undefined ){
        this.readInitialData(update.bind(this));
    }else{
      update();
    }
}

SelectionDesignPoint.prototype.initComponents = function(attributes){
    this.dSelf = this.dParent.selectAll("#" + this.id).data([""]).enter()
        .append("div")
        .attr("id", this.id);

    this.dSelect = this.dSelf.selectAll("#" + this.id + "_select").data([""]).enter()
        .append("select")
        .attr(attributes)
		.attr("class", "designpoint_select")
        .attr("id", this.id + "_select")
        .on("change", this.selectChange.bind(this));
    this.eSelect = document.getElementById(this.id + "_select");

    this.dInfo = this.dSelf.selectAll("#" + this.id + "_info").data([""]).enter()
        .append("div")
        .attr("id", this.id + "_info")
        .style("outline", "solid 1px black");
}

SelectionDesignPoint.prototype.updateDesignPoints = function(callBack){
    var update = function(){
        designPoints = this.dSelect.selectAll("option").data(this.designPointList);
        designPoints.enter().append("option");
        designPoints.attr("designPoint", function(d) { return d })
            .text(function(d) { return "DesignPoint " + d });
        designPoints.exit().remove();

        this.updateDesignPointInfo();
        
        if(callBack != undefined){ callBack() }
    }.bind(this);
    this.readData(update.bind(this));
}

SelectionDesignPoint.prototype.updateDesignPointInfo = function(){
    var designPoint = this.designPointFactors[this.getSelectedDesignPoint()];
    //console.log(designPoint);

    var lines = this.dInfo.selectAll(".key_line").data(Object.keys(designPoint))
    lines.enter().append("div");
    lines.attr("class", "key_line").text(function(d) { return d + " : " + designPoint[d] });
    lines.exit().remove();
    lines.sort();
}

SelectionDesignPoint.prototype.selectChange = function(){
    this.updateDesignPointInfo();
    this.callBack(this.getSelectedDesignPoint());
}

SelectionDesignPoint.prototype.getSelectedDesignPoint = function(){
    designpoint_id = this.eSelect.options[this.eSelect.selectedIndex].getAttribute("designpoint");
    return designpoint_id;
}

SelectionDesignPoint.prototype.addSelectionParameter = function(key, parameter){
    this.parameters[key] = parameter;
}

SelectionDesignPoint.prototype.removeSelectionParameter = function(key){
    delete this.parameters[key];
}

SelectionDesignPoint.prototype.getSelectionParameters = function(){
    parameters = [];
    Object.keys(this.parameters).forEach(function(parameterKey){
        parameters.push(this.parameters[parameterKey]);
    }.bind(this));
    return parameters;
}

//----------------------------------------------
// Run Selection Object Prototypes
// ---------------------------------------------

selectionRunNumber = 0;
SelectionRun = function(exp_id, changeCallBack, parentDomId){
    this.exp_id = exp_id;
    this.callBack = changeCallBack;
    this.dParent = d3.select("#" + parentDomId);
    this.eParent = document.getElementById(parentDomId);
    this.runsByDesignPoint = {};
	this.id = "selectionRun_" + selectionRunNumber;
	selectionRunNumber += 1;
}

SelectionRun.prototype.readData = function(designPoint_id, callBack){
    d3.json("/safe/designpoint/" + designPoint_id + "/simruns/", function(error, data){
        if(error) { console.warn(error) }

        this.runsByDesignPoint[designPoint_id] = data;
        callBack();
    }.bind(this));
}

SelectionRun.prototype.initComponents = function(attributes){
    this.dSelect = this.dParent.selectAll("#" + this.id).data([""]).enter()
        .append("select")
        .attr(attributes)
		.attr("class", "run_select")
		.attr("id", this.id)
        .on("change", this.selectChange.bind(this));
    this.eSelect = document.getElementById(this.id);
}

SelectionRun.prototype.updateRuns = function(designPoint_id, callBack){
    var update = function(){
        runs = this.dSelect.selectAll("option").data(this.runsByDesignPoint[designPoint_id]);
        runs.enter().append("option");
        runs.attr("run", function(d) { return d.simrun_id }).text(function(d) { return "Run " + d.simrun_id });
        runs.exit().remove();
        callBack();
    }.bind(this);
    
    if(this.runsByDesignPoint[designPoint_id] == undefined){
        this.readData(designPoint_id, update);
    }else{
        update();
    }
}

SelectionRun.prototype.selectChange = function(){
    this.callBack(this.getSelectedRun());
}

SelectionRun.prototype.getSelectedRun = function(){
    return this.eSelect.options[this.eSelect.selectedIndex].getAttribute("run");
}

//----------------------------------------------
// Metric Selection Object Prototypes
// ---------------------------------------------

selectionMetricNumber = 0;
SelectionMetric = function(exp_id, changeCallBack, parentDomId){
    this.exp_id = exp_id;
    this.callBack = changeCallBack;
    this.dParent = d3.select("#" + parentDomId);
    this.eParent = document.getElementById(parentDomId);
    this.metricList = {};
    this.colorList = {};
    this.distinctionId = undefined;
	this.id = "selectionMetric_" + selectionMetricNumber;
	selectionMetricNumber += 1;
}

SelectionMetric.prototype.readInitialData = function(callBack){
    d3.json("/safe/exp/" + this.exp_id + "/metrics/", function(error, data){
        if(error) { console.warn(error) }

        data.forEach(function(metric){
            this.metricList[metric.id] = metric;
        }.bind(this));
        callBack();
    }.bind(this));
}

SelectionMetric.prototype.initComponents = function(){
	this.dSelect = this.dParent.selectAll("#" + this.id).data([""]).enter()
		.append("div")
		.attr("id", this.id)
        .style("outline", "solid 1px black");
	this.eSelect = document.getElementById(this.id);

    select_items = this.dSelect.selectAll(".key_line").data(Object.keys(this.metricList)).enter()
        .append("div")
        .attr("class", "select_metric_line")
        .attr("metric_id", function(d) { return this.metricList[d].id }.bind(this))
        .on("click", this.selectChange.bind(this));
        

    select_items.append("div")
        .attr("class", function(d) { return "select_metric_square select_metric_square_" + d }.bind(this))
        .attr("metric_id", function(d) { return d }.bind(this));

    select_items.append("div")
        .attr("class", "select_metric_label")
        .attr("metric_id", function(d) {  return d }.bind(this))
        .text(function(d) { return this.metricList[d].name}.bind(this));

}

SelectionMetric.prototype.setMetricColor = function(distinctionId, metric_id, color){
    if(this.colorList[distinctionId] == undefined) { this.colorList[distinctionId] = {} }
    this.colorList[distinctionId][metric_id] = color;
    this.updateSelectedMetrics();
}

SelectionMetric.prototype.updateSelectedMetrics = function(){
    //console.log("Distinction: ", this.distinctionId, " Colors: ", this.colorList);
    Object.keys(this.metricList).forEach(function(metric_id){
        if(this.colorList[this.distinctionId] != undefined && this.colorList[this.distinctionId][metric_id] != undefined){
            this.dParent.selectAll(".select_metric_square_" + metric_id)
                .style("background", this.colorList[this.distinctionId][metric_id]);
        }else{
            this.dParent.selectAll(".select_metric_square_" + metric_id)
                .style("background", "white");
        }
    }.bind(this));
}

SelectionMetric.prototype.setDistinctionId = function(distinctionId){
    this.distinctionId = distinctionId;
    this.updateSelectedMetrics();
}

SelectionMetric.prototype.selectChange = function(){
    var metric_id = d3.event.toElement.getAttribute("metric_id");
    this.callBack(this.metricList[metric_id]);
}

SelectionMetric.prototype.getMetric = function(metric_id){
    return this.metricList[metric_id];
}

//----------------------------------------------
// Filter selection Object Prototypes
// ---------------------------------------------

selectionFilterNumber = 0;
SelectionResultsFilter = function(parentDomId){
    this.exp_id = exp_id;
    this.dParent = d3.select("#" + parentDomId);
    this.eParent = document.getElementById(parentDomId);
	this.id = "selectionFilter_" + selectionFilterNumber;
	selectionFilterNumber += 1;
    console.log(this.id);
}

SelectionResultsFilter.prototype.initComponents = function(){

	this.dSelect = this.dParent.selectAll("#" + this.id).data([""]).enter()
		.append("div")
		.attr("id", this.id)
        .style("outline", "solid 1px black");
	this.eSelect = document.getElementById(this.id);

    var data = [{name: "Filter Step:", id: this.id + "_filter_step"},
                {name: "Max Results:", id: this.id + "_max_results"},
                {name: "Low Pass Filter", id: this.id + "_denoise_window"}];

    var lines = this.dSelect.selectAll(".key_line").data(data);

    lines = lines.enter().append("div").attr("class", "key_line");

    lines
        .append("div")
        .attr("class", "half_line")
        .text(function(d) { return d.name });

    lines
        .append("input")
        .attr("type", "number")
        .attr("min", 0)
        .attr("value", 0)
        .attr("class", "bordered_half_line")
        .attr("id", function(d) { return d.id });

    this.clusterSize = document.getElementById(this.id + "_filter_step");
    this.maxResults = document.getElementById(this.id + "_max_results");
    this.denoiseWindowSize = document.getElementById(this.id + "_denoise_window");
}

SelectionResultsFilter.prototype.setMetricColor = function(distinctionId, metric_id, color){
    if(this.colorList[distinctionId] == undefined) { this.colorList[distinctionId] = {} }
    this.colorList[distinctionId][metric_id] = color;
    this.updateSelectedMetrics();
}

SelectionResultsFilter.prototype.getClusterSize = function(){
    if (this.clusterSize.value == ""){ this.clusterSize.value = 0 }
    return this.clusterSize.value;
}

SelectionResultsFilter.prototype.getMaxResults = function(){
    if (this.maxResults.value == ""){ this.maxResults.value = 0 }
    return this.maxResults.value;
}

SelectionResultsFilter.prototype.getDenoiseWindowSize = function(){
    if (this.denoiseWindowSize.value == ""){ this.denoiseWindowSize.value = 0 }
    return this.denoiseWindowSize.value;
}

SelectionResultsFilter.prototype.getResultsParameters = function(){
    return {"filter_step": this.getClusterSize(),
            "max_results":this.getMaxResults(),
            "denoise_window":this.getDenoiseWindowSize()};
}


//----------------------------------------------
// Plot Area Object Object
// ---------------------------------------------

graphOptionsNumber = 0;
GraphOptions = function(exp_id, optionChangeCallBack, parentDomId){
	this.exp_id = exp_id;
	this.optionChangeCallBack = optionChangeCallBack;
	this.id = parentDomId + "_optionsPlotArea_" + optionsPlotAreaNumber;
	selectionPlotAreaNumber += 1;

	this.dSelf = d3.select("#" + parentDomId).append("div")
        .attr("id", this.id)
        .attr("class", "plot_area_options");
	this.eSelf = document.getElementById(this.id);

    //console.log("Options: ", this.dSelf);
    this.initComponents();
}


GraphOptions.prototype.initComponents = function(){
    var id = ["x_axis_label", "y_axis_label"];
    var labelData = ["X Axis Label:", "Y Axis Label:"];

    var labels = this.dSelf.selectAll("div").data(labelData).enter()
        .append("div");

    labels.append("div")
        .attr("class", "select_line")
        .text(function(d) { return d });

    labels.append("input")
        .attr("class", "select_line")
        .attr("id", function(d, e) { return this.id + id[e] }.bind(this)) 
        .on("change", this.labelChange.bind(this));

    this.dSelf.append("div").attr("class", "select_line").text("Move Selected Graph:");
    var position = this.dSelf.append("div").attr("class", "select_line");
    position.append("button").attr("class", "half_line").text("UP").on("click", this.moveSelectedUp.bind(this));
    position.append("button").attr("class", "half_line").text("DOWN").on("click", this.moveSelectedDown.bind(this));

    this.xAxisLabel = document.getElementById(this.id + id[0]);
    this.yAxisLabel = document.getElementById(this.id + id[1]);
}

GraphOptions.prototype.setSelectedGraph = function(graph){
    this.selectedGraph = graph;
    this.setLabels();
}

GraphOptions.prototype.setContextGraph = function(graph){
    this.contextGraph = graph;
}

GraphOptions.prototype.labelChange = function(){
    if(this.selectedGraph){
        this.selectedGraph.setXAxisLabel(this.xAxisLabel.value);
        this.selectedGraph.setYAxisLabel(this.yAxisLabel.value);
    }
    if(this.contextGraph){
        this.contextGraph.setXAxisLabel(this.xAxisLabel.value);
        this.contextGraph.setYAxisLabel(this.yAxisLabel.value);
    }
}

GraphOptions.prototype.setLabels = function(){
    if(this.selectedGraph){
        this.xAxisLabel.value = this.selectedGraph.getXAxisLabel();
        this.yAxisLabel.value = this.selectedGraph.getYAxisLabel();
    }
    if(this.contextGraph){
        this.contextGraph.setXAxisLabel(this.selectedGraph.getXAxisLabel());
        this.contextGraph.setYAxisLabel(this.selectedGraph.getYAxisLabel());
    }
}

GraphOptions.prototype.moveSelectedUp = function(){
    if(!(this.contextGraph && this.selectedGraph)){ return }

    var selected = this.selectedGraph.svg[0][0];
    var context = this.contextGraph.svg[0][0];
    
    selected.parentNode.insertBefore(selected, selected.previousSibling);
    context.parentNode.insertBefore(context, context.previousSibling);
}

GraphOptions.prototype.moveSelectedDown = function(){
    if(!(this.contextGraph && this.selectedGraph)){ return }
 

    var selected = this.selectedGraph.svg[0][0];
    var context = this.contextGraph.svg[0][0];
    
    if(context.nextSibling){ 
        context.parentNode.insertBefore(context, context.nextSibling.nextSibling);
        selected.parentNode.insertBefore(selected, selected.nextSibling.nextSibling);
    }else{
        context.parentNode.insertBefore(context, context.parentNode.firstChild);
        selected.parentNode.insertBefore(selected, selected.parentNode.firstChild);
    }
}
