// Initializes the selection for the graph
// callBack of the form callBack(run_id, metric_id)
function SelectionFull(exp_id, selectionCallBack, applyFilterCallBack, parentDomId){
    this.exp_id = exp_id;
    this.dParent = d3.select("#" + parentDomId);
    console.log(this.dParent, parentDomId);
    this.eParent = document.getElementById(parentDomId);
    this.selectionCallBack = selectionCallBack;
    this.applyFilterCallBack = applyFilterCallBack;
    this.parameterList = {};
    this.displayList = {};
    this.readInitialData();
}
    
SelectionFull.prototype.initComponents = function(){
    // Initialize the factor list
    this.dSelectFactor = this.dParent.selectAll(".selectFactor").data([""]).enter()
        .append("div")
        .append("select")
        .attr("class", "key_line selectFactor")
        .on("change", this.factorSelectChange.bind(this));
    this.eSelectFactor = this.eParent.getElementsByClassName("selectFactor")[0];

    this.dSelectFactor
        .selectAll("option")
        .data(Object.keys(this.factorList))
        .enter()
        .append("option")
        .attr("factor", function(d) { return d } )
        .text(function(d) { return this.factorList[d].name }.bind(this));
    
    // Initialize the level list
	var rangeDiv = this.dParent.append("div")

    this.dSelectLevelMin = rangeDiv.selectAll(".selectLevelMin").data([""]).enter()
        .append("select")
		.style({"display": "inline-block", "width":"50%"})
        .attr("class", "selectLevelMin");
    this.eSelectLevelMin = this.eParent.getElementsByClassName("selectLevelMin")[0];

    this.dSelectLevelMax = rangeDiv.selectAll(".selectLevelMax").data([""]).enter()
        .append("select")
		.style({"display": "inline-block", "width":"50%"})
        .attr("class", "selectLevelMax");
    this.eSelectLevelMax = this.eParent.getElementsByClassName("selectLevelMax")[0];
    
    // Initialize the apply selection button
    this.dParent    
        .append("button")
        .text("Apply Selection Filter")
        .style("width","100%")
        .on("click", this.applySelectionFilter.bind(this));
    
    
    // Initialize the design point list
    this.dSelectDesignPoint = this.dParent.selectAll(".selectDesignPoint").data([""]).enter()
        .append("div")
        .append("select")
        .attr("class", "key_line selectDesignPoint")
        .on("change", this.designPointSelectChange.bind(this));
    this.eSelectDesignPoint = this.eParent.getElementsByClassName("selectDesignPoint")[0];
    console.log(this.eSelectDesignPoint);

    // Initialize the run list
    this.dSelectRun = this.dParent.selectAll(".selectRun").data([""]).enter()
        .append("div")
        .append("select")
        .attr("class", "key_line selectRun")
        .on("change", this.runSelectChange.bind(this));
    this.eSelectRun = this.eParent.getElementsByClassName("selectRun")[0];
    
    // Initialize metric selects
    select_items = this.dParent.selectAll(".select_line").data(Object.keys(this.metricList)).enter()
        .append("div")
        .attr("class", "select_line")
        .attr("metric_id", function(d) { return this.metricList[d].id }.bind(this))
        

    select_items.append("div")
        .attr("class", function(d) { return "select_square select_square_" + this.metricList[d].id }.bind(this))
        .attr("metric_id", function(d) { return this.metricList[d].id }.bind(this));

    select_items.append("div")
        .attr("class", "select_label")
        .attr("metric_id", function(d) {  return this.metricList[d].id }.bind(this))
        .text(function(d) { return this.metricList[d].name}.bind(this));

    this.dParent.selectAll(".select_line")
        .on("click", this.selectElementChange.bind(this));


    this.factorSelectChange();
    this.updateDesignPoints();
}

// Reads metric information
SelectionFull.prototype.readInitialData = function(){

    d3.json("/safe/exp/" + exp_id + "/designpoints/", function(error, data){
        if(error){ console.warn(error) }

        this.designPointList = {};
        this.runList = [];

        data.forEach(function(designPoint){ this.designPointList[designPoint.id] = designPoint}.bind(this));
        data.forEach(function(designPoint){ d3.json("/safe/designpoint/" + designPoint.id + "/simruns/", function(error, data){
                if(error) { console.warn(error) }

                this.runList.concat(data);
                designPoint["runs"] = data;
                }.bind(this));
            }.bind(this));

        d3.json("/safe/exp/" + exp_id + "/factors/", function(error, data){
            if(error){ console.warn(error) }
            
            this.factorList = {};
            this.levelList = {};
            data.forEach(function(factor){ this.factorList[factor.id] = factor }.bind(this));

            d3.json("/safe/exp/" + exp_id + "/levels/", function(error, data){
                if(error){ console.warn(error) }

                Object.keys(this.factorList).forEach(function(factor_id){
                    this.levelList[factor_id] = data[factor_id].levels.sort();
                  }.bind(this));

                this.metricList = {};
                d3.json("/safe/exp/" + exp_id + "/metrics/", function(error, data){
                    if(error){ console.warn(error) }
                    data.forEach(function(element){
                        this.metricList[element.id] = element;
                      }.bind(this));
                    this.initComponents();
                  }.bind(this));
              }.bind(this));
            
        }.bind(this));
    }.bind(this));
}


SelectionFull.prototype.factorSelectChange = function(){
    console.log("Factor changed...");
    console.log(this);

	levels = this.dSelectLevelMin.selectAll("option").data(this.levelList[this.getSelectedFactor()]);
	levels.enter().append("option");
	levels.attr("level", function(d) { return d }).text(function(d) { return d });
	levels.exit().remove();

    levels = this.dSelectLevelMax.selectAll("option").data(this.levelList[this.getSelectedFactor()]);
	levels.enter().append("option");
	levels.attr("level", function(d) { return d }).text(function(d) { return d });
	levels.exit().remove();
	this.eSelectLevelMax.selectedIndex = this.levelList[this.getSelectedFactor()].length - 1;

}
SelectionFull.prototype.applySelectionFilter = function(){
    console.log("Level changed...");

    if(this.getMinLevel() > this.getMaxLevel()){
        alert("The factor's beginning value cannot be greater than the factor's ending value");
		return;
    }

    factor = this.factorList[this.getSelectedFactor()];
    min = this.levelList[this.getSelectedFactor()][0];
    max = this.levelList[this.getSelectedFactor()][this.levelList[this.getSelectedFactor()].length - 1];
    console.log(min, max);
	if(this.getMinLevel() > min || this.getMaxLevel() < max ){
		this.parameterList[factor.id] = {"name": factor.name, "factor_id": factor.id, "min_value":this.getMinLevel(), "max_value":this.getMaxLevel()};
	}else{
		delete this.parameterList[factor.id];
	}

    this.updateDesignPoints();
    this.applyFilterCallBack();
}

SelectionFull.prototype.removeSelectionFilter = function(factor_id){
    console.log("Removing ", factor_id);
    delete this.parameterList[factor_id];
    this.updateDesignPoints();
    this.applyFilterCallBack();
}

SelectionFull.prototype.updateDesignPoints = function(){
	parameters = this.getFilterParameters();
    console.log(parameters);

    d3.json("/safe/exp/" + exp_id + "/dp-range-query/")
      .post(JSON.stringify(parameters), function(error, data){
        if(error){ console.ward(error); }
            
        this.dSelectDesignPoint.selectAll("option").remove();
        var designPoints = this.dSelectDesignPoint.selectAll("option").data(data);
        designPoints.enter().append("option");
        designPoints.attr("designPoint", function(d) { return d }).text(function(d) { return "Design Point " + d});
        this.designPointSelectChange();
      }.bind(this));
}


SelectionFull.prototype.designPointSelectChange = function(){
    console.log("Design Point changed...");
    d3.json("/safe/designpoint/" + this.getSelectedDesignPoint() + "/simruns/", function(error, data){
        if(error){ console.warn(error) }

        runs = this.dSelectRun.selectAll("option").data(data);
        runs.enter().append("option")
        runs.attr("run", function(d) { return d.simrun_id }).text(function(d) { return "Run " + d.simrun_id });
        runs.exit().remove();
        // Prepend aggregate run here
        }.bind(this));
}

SelectionFull.prototype.runSelectChange = function(){
    console.log("Run changed...");
    this.updateSelectSquares();
}

SelectionFull.prototype.updateSelectSquares = function(){
    var run_id = this.getSelectedRun();
    Object.keys(this.metricList).forEach(function(metric_id){
        if(this.displayList[run_id] != undefined && this.displayList[run_id][metric_id] != undefined){
            this.dParent.selectAll(".select_square_" + metric_id).style("background", this.displayList[run_id][metric_id]);
        }else{
            this.dParent.selectAll(".select_square_" + metric_id).style("background", "white");
        }
        }.bind(this));
}

SelectionFull.prototype.addDisplayed = function(run_id, metric_id, color){
    if(this.displayList[run_id] == undefined){ this.displayList[run_id] = {} }
    this.displayList[run_id][metric_id] = color;
    this.updateSelectSquares();
}

SelectionFull.prototype.getFilterParameters = function(){
    parameters = [];
	Object.keys(this.parameterList).forEach(function(parameter){ parameters.push(this.parameterList[parameter]) }.bind(this));
    return parameters;
}

SelectionFull.prototype.removeDisplayed = function(run_id, metric_id){
    if(this.displayList[run_id] != undefined){ delete this.displayList[run_id][metric_id] }
    this.updateSelectSquares();
}

SelectionFull.prototype.selectElementChange = function(event){
    console.log(d3.event.target, event);
     this.selectionCallBack(this.getSelectedRun(), d3.event.toElement.getAttribute("metric_id"));
}

SelectionFull.prototype.getSelectedFactor = function(){
    return this.eSelectFactor.options[this.eSelectFactor.selectedIndex].getAttribute("factor");
}

SelectionFull.prototype.getMinLevel = function(){
    return this.eSelectLevelMin.options[this.eSelectLevelMin.selectedIndex].getAttribute("level");
}

SelectionFull.prototype.getMaxLevel = function(){
    return this.eSelectLevelMax.options[this.eSelectLevelMax.selectedIndex].getAttribute("level");
}

SelectionFull.prototype.getSelectedDesignPoint = function(){
    return this.eSelectDesignPoint.options[this.eSelectDesignPoint.selectedIndex].getAttribute("designPoint");
}

SelectionFull.prototype.getSelectedRun = function(){
    return this.eSelectRun.options[this.eSelectRun.selectedIndex].getAttribute("run");
}
