// The experiment currently being observed
var exp_id, normalize;
var colors = safe.colors.QUALITATIVE_0;
var focusDataPointRadius = 2;
var contextDataPointRadius = 1.5;
var numberFormat = d3.format(".2f");
var windowResizeFunctions = [];

var selectFactor;


MetricPlotArea = function(exp_id, parentElement){
	this.exp_id = exp_id;
	this.parent = safe.tools.d3Object(parentElement);
    this.margin = {top: 10, right: 40, bottom: 50, left: 60};

	this.colorList = {};		// list of colors by run_id and metric_id
	this.displayList = {};		// list of metrics being displayed by run_id and metric_id
	
    this.graphWidth = undefined;
    this.contextHeight = 180;
    this.focusHeight = 500;

    this.selectedGraph = undefined;
    this.contextGraph = undefined;
    this.currentMetric = undefined;

    this.initComponents();
}

MetricPlotArea.prototype.initComponents = function(){
    this.dSelf = this.parent.append("div").attr("class", "plot_area");
    windowResizeFunctions.push(this.resize.bind(this));

    this.main = this.dSelf.append("div").style({"width":"80%", "display":"inline-block"});
    var title = this.main.append("div").attr("class", "plot_area_label").text("Time Series");
    this.graphArea = this.main.append("div").attr("class", "graph_area");
    this.graphWidth = this.graphArea[0][0].offsetWidth;

    this.select = this.dSelf.append("div").style({"width":"20%", "display":"inline-block", "vertical-align":"top"});
    this.select.append("div").attr("class", "plot_selection_label").text("Select");
	this.selection = new safe.selector.full(this.exp_id, this.selectionChange.bind(this), this.addSelectionParameter.bind(this), this.select);

    this.select.append("div").style("height","0.5cm");
    this.select.append("div").attr("class", "plot_selection_label").text("Options");
    this.options = new safe.selector.options(title, this.select);
    
    this.displayed = this.main.append("div").attr("class", "plot_displayed");
    this.displayed.append("div").attr("class", "plot_displayed_label").text("Displayed Metrics");
    this.displayKey = new safe.infobox.colored(this.removeMetricFromDisplay.bind(this) , this.displayed);

    this.reserved = this.main.append("div").attr("class", "plot_reserved");
    this.reserved.append("div").attr("class", "plot_reserved_label").text("Reserved Colors");
    this.reservedColors = new safe.infobox.colored(this.removeReservedColor.bind(this), this.reserved);

    this.filtered = this.main.append("div").attr("class", "plot_filtered");
    this.filtered.append("div").attr("class", "plot_filtered_label").text("Design Point Filter");
    this.selectParameters = new safe.infobox.default(this.removeSelectionParameter.bind(this), this.filtered);

    this.focus = new safe.graph.timeplot(this.margin, this.graphWidth, this.focusHeight, undefined, this.graphArea, true);
    this.context = new safe.graph.timeplot(this.margin, this.graphWidth, this.contextHeight, undefined, this.graphArea, false);
    this.context.addBrush(this.brushed.bind(this), safe.constants.Y_AXIS);
    this.context.yAxis.ticks(5);
    this.options.setContextGraph(this.context);
    this.options.setSelectedGraph(this.focus);
    this.options.disableGraphMovement();
}

// levelDict -> a dictionary of (level_nkeyString ame, level) pairs
MetricPlotArea.prototype.selectionChange = function(run_id, metric){
    var metric_id = metric.id;

    if(this.currentMetric == undefined){ this.currentMetric = metric }
    if(metric.id != this.currentMetric.id){ 
        alert("Cannot display data from multiple metrics on the same plot. The current metric type is " + this.currentMetric.name + ".");
        return;
    }

    if(this.colorList[run_id] == undefined){ this.colorList[run_id] = {} }
	if(this.displayList[run_id] == undefined) { this.displayList[run_id] = {} }

	if(this.colorList[run_id][metric_id] == undefined) { 
        if(colors.length == 0){
            alert("All available colors have been used. Please remove one or more colors from the reserved colors list for further use.");
            return;
        }
        this.colorList[run_id][metric_id] = colors.shift() 
    }else{
        this.reservedColors.removeElement(run_id + "_" + metric_id);
    }

	if(this.displayList[run_id][metric_id] == undefined){
		this.addDisplayMetric(run_id, metric);
	}else{
		this.removeDisplayMetric(run_id, metric);
	}
}

MetricPlotArea.prototype.addDisplayMetric = function(run_id, metric){
    metric_id = metric.id;
    this.displayList[run_id][metric_id] = true;

    var id = run_id + "_" + metric_id;

	var update = function(){
        this.options.setSelectedGraph(this.focus);
        this.focus.display(id);
        this.context.display(id);
        this.selection.addDisplayed(run_id, metric_id, this.colorList[run_id][metric_id]);

        var displayString = "Run " + run_id + ", Metric: " + metric.name;
        var color = this.colorList[run_id][metric_id];
        var returnAttributes = {"run_id":run_id, "metric":metric };

        this.displayKey.addElement(id, displayString, color, returnAttributes)
        this.displayKey.updateElements();
    }.bind(this)

    var addPlot = function(data){
        pointAttributes = {"stroke": this.colorList[run_id][metric_id], 
                           "stroke-width": 1.5, 
                           "fill": "white", 
                           "fill-opacity": 0, 
                           "r": focusDataPointRadius};
        
        this.context.addDataSet(id, data, function(d){ return d.time }, function(d){ return d.value }, pointAttributes);
        this.focus.addDataSet(id, data, function(d){ return d.time }, function(d){ return d.value }, pointAttributes);
        this.focus.setYAxisLabel(metric.name + "(" + metric.units + ")");
        this.focus.setXAxisLabel("Time (seconds)");
    }.bind(this)


    if(!this.focus.hasDataSet(id)){
        var parameters = this.formatResultsParameters(this.selection.getResultsParameters()); 

        d3.json("/safe/simrun/" + run_id + "/metric/" + metric_id + "/results/" + parameters, function(error, data){
            if(error){ console.warn(error) }
            data.sort(function(a, b){ return a.time - b.time });

            addPlot(data);
			update();
        }.bind(this));
    }else{
        this.focus.setVisible(id);
        this.context.setVisible(id);
		update();
    }
}

MetricPlotArea.prototype.removeMetricFromDisplay = function(attributes){
    this.removeDisplayMetric(attributes.run_id, attributes.metric);
}

MetricPlotArea.prototype.removeDisplayMetric = function(run_id, metric){
    var metric_id = metric.id;
    var id = run_id + "_" + metric_id;
    var displayString = "Run " + run_id + ", Metric: " + metric.name;
    var returnAttributes = {"run_id":run_id, "metric":metric};

    this.displayKey.removeElement(id);
    this.reservedColors.addElement(id, displayString, this.colorList[run_id][metric_id], returnAttributes);
    this.selection.removeDisplayed(run_id, metric_id);
    this.focus.remove(id);
    this.context.remove(id);

    delete this.displayList[run_id][metric_id];

    if(this.focus.getVisibleIds().length == 0){ this.currentMetric = undefined }
}

MetricPlotArea.prototype.removeSelectionParameter = function(parameter){
    console.log("Removing Selection: ", parameter);
    this.selectParameters.removeElement(parameter.factor_id);
    this.selection.removeDesignFilter(parameter.factor_id);
}

MetricPlotArea.prototype.addSelectionParameter = function(parameter){
    console.log("Adding Selection: ", parameter);
    var id = parameter.factor_id;
    var displayString = "Factor: " + parameter.name + " , Range: "+ parameter.min_value + " - " + parameter.max_value;
    var returnAttributes = parameter;
    this.selectParameters.addElement(id, displayString, returnAttributes);
}

MetricPlotArea.prototype.removeReservedColor = function(attributes){
    var run_id = attributes.run_id;
    var metric_id = attributes.metric.id;
    console.log(attributes, this.colorList[run_id][metric_id]);

    colors.push(this.colorList[run_id][metric_id]);
    delete this.colorList[run_id][metric_id];
    this.reservedColors.removeElement(run_id + "_" + metric_id);
}

MetricPlotArea.prototype.onGraphSelect = function(graph, src){
    if(this.selectedGraph)
        this.selectedGraph.svg.style("border", "none");
    graph.svg.style({"border": "solid 1px red"});

    this.selectedGraph = graph;
    this.context.removeAll();

    var id = graph.getVisibleIds()[0];
    var dataSet = graph.getDataSet(id);
    var pointAttributes = graph.getPointAttributes(id);
    
    this.context.addDataSet(id, dataSet, pointAttributes);
    this.context.display(id);

    this.options.setSelectedGraph(graph);
    if(src == undefined){ src = d3.event.target }
    var cxt = this.context.svg[0][0];
    cxt.parentNode.insertBefore(cxt, src.nextSibling);
}

MetricPlotArea.prototype.brushed = function(brush) {
    var extent = brush.extent();
    xDomain = brush.empty() ? this.context.xScale.domain() : [ extent[0], extent [1] ];

    this.focus.setXDomain(xDomain);
    this.focus.displayAll();
}

MetricPlotArea.prototype.resize = function(){
    var graphArea = this.main[0][0];
    if(graphArea == undefined){ return }

    this.graphWidth = graphArea.offsetWidth;
    this.context.setWidth(this.graphWidth);
    this.context.displayAll();

    this.focus.setWidth(this.graphWidth);
    this.focus.displayAll();
}

MetricPlotArea.prototype.formatResultsParameters = function(parameters){
    var parameterString = "?";
    Object.keys(parameters).forEach(function(key, index){
          if(index > 0){ parameterString += "&" }
          parameterString += key;
          parameterString += "=" + parameters[key];
    });
    return parameterString;
}

//======================================================

function onWindowResize(){
    windowResizeFunctions.forEach(function(resizeFunction){
        resizeFunction();
    });
}

function initVisualComponents(){
    d3.select(window).on("resize", onWindowResize);
	new MetricPlotArea(exp_id, "place_graphs")}

function addFactorPlotArea(factor){
	//console.log("Factor Changed: ", factor);
	new MetricPlotArea(exp_id, factor, "place_graphs");
    d3.select("#place_graphs").append("div").attr("class", "graph_sep");
}

function readParameters(){
    var parameterList = location.search.substring(1).split("&");
    var parameters = {};
    parameterList.forEach(function(parameter){
        var temp = parameter.split("=");
        parameters[temp[0]] = temp[1];
        });

    exp_id = parameters["exp_id"];
    normalize = parameters["normalize"] == undefined ? false: parameters["normalize"] == "true";
}

readParameters();
initVisualComponents();
