/**-------------------------------------
 * A factor selector object
 * -------------------------------------
 *
 * An object prototype which allows the selection of factors
 * which are stored in the database.
 */
safe.selector.factor = safe_selector_factor;

/**
 * Creates a new factor selector object
 * @param exp_id The experiment id with which this selector is associated
 * @param changeCallBAck a function to be called when the selection is changed
 * @param parentElement The dom element, dom id, or d3 object to which this
 *        selector should be added.
 */
function safe_selector_factor(exp_id, changeCallBack, parentElement){
    this.exp_id = exp_id;
    this.callBack = changeCallBack;
    this.dParent = safe.tools.d3Object(parentElement);
    this.factorList = {};
}

/**
 * Reads data from the database about available factors
 * @param callBack A function to be called when the read is complete
 */
safe_selector_factor.prototype.readInitialData = function(callBack){
    d3.json("/safe/exp/" + this.exp_id + "/factors/", function(error, data){
        if(error){ console.warn(error) }

        data.forEach(function(factor){ this.factorList[factor.id] = factor }.bind(this));
        if(callBack != undefined){ callBack() }
    }.bind(this));
}

/**
 * Initializes the visual dom elements for this selector
 */
safe_selector_factor.prototype.initComponents = function(attributes, style){
    this.dSelect = this.dParent
        .append("select")
        .attr("class", "form-control")
        .style(style)
        .on("change", this.selectChange.bind(this));
    this.eSelect = this.dSelect[0][0];

    this.dSelect
        .selectAll("option")
        .data(Object.keys(this.factorList))
        .enter()
        .append("option")
        .attr("factor", function(d) { return d })
        .text(function(d) { return this.factorList[d].name }.bind(this));
}

/**
 * Called when there is a change in the currently selected factor.
 */
safe_selector_factor.prototype.selectChange = function(){
    if(this.callBack != undefined){ this.callBack(this.getSelectedFactor()) }
}

/**
 * Returns the factor that is currently selected
 * @return The currently selected factor
 */
safe_selector_factor.prototype.getSelectedFactor = function(){
    var factor_id = this.eSelect.options[this.eSelect.selectedIndex].getAttribute("factor");
    return this.factorList[factor_id];
}
