/**-------------------------------------
 * A file selector object 
 * -------------------------------------
 *
 * An object that allows the user to select a file from
 * their file system
 */

safe.selector.file = safe_selector_file;

/**
 * Creates a new file selector object
 *
 * @param parentElement The dom element, d3 object, or dom id
 *        to which this selector should be added
 */
function safe_selector_file(parentElement){
    this.parent = safe.tools.d3Object(parentElement);
    this.data = {};
}

/** Initializes the dom components of this selector */
safe_selector_file.prototype.initComponents = function(){
    this.dSelf = this.parent.append("div");

    var file = this.dSelf.append("input")
        .attr("type", "file")
        .attr("name", "file")
        .style("width", "100%");

    var progress = this.dSelf.append("div");

    var list = this.dSelf.append("output");

    var div = this.dSelf.append("div")
        .attr("class", "file_name_select");
     
    div.append("label")
        .attr("class", "half_line")
        .text("Distribution Name:");
    
    var fileName = div.append("input")
        .attr("type", "text")
        .attr("class", "bordered_half_line");
        
    this.dSelf.append("button")
        .attr("class", "upload_button")
        .text("Upload")
        .on("click", this.uploadFile.bind(this)); 

    this.fileName = fileName[0][0];

    this.file = file[0][0];
    this.list = list[0][0];
    this.progress = progress[0][0];

    this.file.addEventListener("change", this.fileSelectChange.bind(this), false);
}

/** called when there is a change in the selected file */
safe_selector_file.prototype.fileSelectChange = function(event){
    var files = this.file.files;
    var output = [];

    for(var i = 0, f; f = files[i]; i++){

        var reader = new FileReader();

        reader.onload = this.loadedListener.bind(this);
        reader.onprogress = this.progressListener.bind(this);

        reader.readAsText(f);

        output.push("<li><strong>", escape(f.name), 
                    "</strong> (",
                    f.type || "n/a",
                    ") - ",
                    f.size,
                    " bytes, last modified: ",
                    f.lastModifiedDate ? f.lastModifiedDate.toLocaleDateString() : "n/a",
                    "</li>");
    }
    this.list.innerHTML = "<ul>" + output.join("") + "</ul>";
}

/** Listens for progress in the file upload */
safe_selector_file.prototype.progressListener = function(event){
    if(event.lengthComputable){
        var percent = event.loaded / event.total;
        
        while(this.progress.hasChildNodes()){
            this.progress.removeChild(this.progress.firstChild);
        }

        this.progress.appendChild(document.createTextNode(Math.round(percent * 100) + "%"));
    }
}

/** Listens for the completion of a file load */
safe_selector_file.prototype.loadedListener = function(event){
    this.fileData = event.target.result;
    this.progress.style.display = "none";
}

/** Listens for errors in the upload of the file */
safe_selector_file.prototype.errorListener = function(event){
    alert("Upload failed");
}

/** Returns the file object that has been uploaded */
safe_selector_file.prototype.getFile = function(){
    return this.file.files[0];
}

/** Returns the data of the uploaded file */
safe_selector_file.prototype.getFileData = function(){
    return this.fileData;
}

/**
 * Uploads as a distribution the file stored in the file selector as a distribution
 */
safe_selector_file.prototype.uploadFile = function(){
    var file = this.getFile();
    var data = this.getFileData();

    var match = file.type.match(".*(csv|json)");
    if(match != null && match[1] == "csv" || match == null && file.name.match(".*\.csv")){
        data = d3.csv.parse(data);
    }else if(match != null && match[1] == "json" || match == null && file.name.match(".*\.json")){
        data = JSON.parse(data);
    }else{
        alert("Invalid file type. Must be .csv or .json");
    }

    console.log(this.fileName.value, data);
}
