//-------------------------------------
// Hider Button
//-------------------------------------
//
// An object that can be added to an html element
// which creates minimize and maximize button which
// control whether the element is fully displayed
// or hidden.

safe.tools.hider = safe_tools_hider;

/**
 * Create a new hider object
 * @param element The element to be hidden
 */
function safe_tools_hider(element){
    this.element = safe.tools.d3Object(element);
}

/**
 * Initialize the hider components within the element
 */
safe_tools_hider.prototype.initComponents = function(){
    this.displayed = this.element.append("div")
        .style({"width": "100%", "padding": "0px" });

    this.displayed.append("i")
        .attr("class", "glyphicon glyphicon-minus-sign")
        .style({"cursor": "pointer", "padding-left": "0.5cm", "padding-right": "0.2cm" })
        .on("click", this.toggleHidden.bind(this));

    this.displayed.append("label").text(" Hide");

    var domElement = this.element[0][0];
    var newDiv = document.createElement("div");
    var hidden = domElement.parentNode.insertBefore(newDiv, domElement);

    this.hidden = d3.select(newDiv)
        .attr("class", "well container")
        .style({"width": "95%", "display": "none"});

    this.hidden.append("i")
        .attr("class", "glyphicon glyphicon-plus-sign")
        .style({"cursor": "pointer", "padding-left": "0.5cm", "padding-right": "0.2cm"})
        .on("click", this.toggleHidden.bind(this));

    this.hidden.append("label").text("Show");
}

/**
 * Toggles whether the element is hidden or displayed
 */
safe_tools_hider.prototype.toggleHidden = function(){
    var temp = this.element.style("display");
    this.element.style("display", this.hidden.style("display"));
    this.hidden.style("display", temp);
}
